#Electro-L import routine
#import of all spectral channels, result of channel 11.9 needs further validation
from tkinter import *
from tkinter.filedialog import askdirectory
from tkinter.filedialog import askopenfilename
from tkinter.ttk import Progressbar, Style
import os
import sys
import shutil
from threading import Thread
import subprocess

from satpy.scene import Scene
from satpy.writers import get_enhanced_image
import glob
from osgeo import gdal
import ilwis
import numpy as np
import zipfile

#passing parameters from sub menu

def execute(GNCDir, InFileName, WorkDir, OutFileName):
    if len(GNCDir) > 0 and len(InFileName) > 0 and len(WorkDir) > 0 and len(OutFileName) > 0:

        progress['value'] = 0
        time_stamp = InFileName
        
        progress['value'] = 10
        
        os.chdir(WorkDir)
        ilwis.setWorkingCatalog(WorkDir)
        
        
        for filename in glob.glob(os.path.join(GNCDir+'/H-000-GOMS3_-GOMS3_4_____-*'+str(time_stamp)+'*')):
            shutil.copy(filename, WorkDir)
        shutil.copy(gnc_path + '/extern/MSG_decomp/xRITDecompress.exe', WorkDir)
        
        progress['value'] = 20

        list = glob.glob(WorkDir+'/H-000-GOMS3_-GOMS3_4_____-*'+str(time_stamp)+'*')
        list = [elem[len(WorkDir)+1:] for elem in list]

        si = subprocess.STARTUPINFO()
        si.dwFlags |= subprocess.STARTF_USESHOWWINDOW
        for elem in list:
            decode = subprocess.run(['xRITDecompress.exe',elem], startupinfo=si)
        
        #load data for selected color visualization
        fnames = glob.glob(WorkDir+'/H-000-GOMS3_-GOMS3_4_____-*'+str(time_stamp)+'*__')
        scn = Scene(reader='electrol_hrit', filenames=fnames)

        progress['value'] =40

        #Import of spectral channels
        scn.load(["00_9"])
        B009_values = scn["00_9"].values

        scn.load(["03_8"])
        B038_values = scn["03_8"].values
        
        scn.load(["08_0"])
        B08_values = scn["08_0"].values
        
        scn.load(["09_7"])
        B097_values = scn["09_7"].values
        
        scn.load(["10_7"])
        B107_values = scn["10_7"].values
        
        scn.load(["11_9"])
        B119_values = scn["11_9"].values
        
        progress['value'] = 60
        
        # create output map in ILWIS format
        data = np.array((B009_values, B038_values, B08_values, B107_values, B119_values))
        
        
        progress['value'] = 80
        
        grf = ilwis.GeoReference('code=georef:type=corners,csy=proj4:+proj=geos +a=6378169 +lon_0=76 +h=35785831 +rf=295.488065897001, envelope=-5566748.0802 5570748.6178 5570748.6178 -5566748.0802, gridsize=2784 2784, cornerofcorners=yes')
        dfNum = ilwis.DataDefinition(ilwis.NumericDomain('code=value'), ilwis.NumericRange(-1000.0, 1000.0, 0.00001))
        rcNew = ilwis.RasterCoverage()
        rcNew.setSize(ilwis.Size(2784,2784,5))
        rcNew.setGeoReference(grf)
        rcNew.setDataDef(dfNum)
        
        rcNew.array2raster(data.flatten())
        
        rcNew = ilwis.do('mapcalc', 'iff(@1>=-100,@1,?)', rcNew)
        rcNew.store(OutFileName+'.mpl')
        
        progress['value'] = 95
        
        #redundant files to be deleted 
        if os.path.exists(WorkDir + '/xRITDecompress.exe'):
            os.remove (WorkDir + '/xRITDecompress.exe')

        #Other redundant files are deleted in main menu script
        progress['value'] = 100
        
        button0.config(state='normal')
        button1.config(state='normal')

def executeInThread(GNCDir, InFileName, WorkDir, OutFileName):
    button0.config(state='disabled')
    button1.config(state='disabled')
    thread = Thread(target = execute, args = (GNCDir, InFileName, WorkDir, OutFileName))
    thread.start()



# for command=doNothing
def doNothing():
   text="Do nothing button"
   return


# for command=clickthumbn
def clickthumbn():
   novi = Toplevel(app)
   novi.title ('Example of visualized Electro-L image (in ILWIS)')
   novi.iconbitmap(gnc_path +'\pics\gnc.ico')
   canvas = Canvas(novi, width = 400, height = 400)
   canvas.pack(expand = YES, fill = BOTH)
   gif1 = PhotoImage(file = gnc_path +'\pics\Electro_L10_7.gif')
   canvas.create_image(0, 0,image = gif1, anchor = NW)
   canvas.gif1 = gif1

# for command=clickAbout
def clickAbout(): 
   filewin = Toplevel(app)
   filewin.title ('About Electro-L')
   filewin.iconbitmap(gnc_path +'\pics\gnc.ico')
   filewin.geometry("575x340")
   msg = Message(filewin, text="""Electro-L background:

The Russian geostationary satellite Electro-L N3 (the third satellite in the series) is situated at 76° East. Of the 10 channels of the MSU-GS imager the following channels are provided every 30 minutes (Channel no, Spectral band [μm]): 3: 0.8 – 0.9, 4: 3.5 – 4.0, 6: 7.5 – 8.5, 8: 9.2 – 10.2, 9: 10.2 – 11.2, 10: 11.2 – 12.5.

File name convertion: H-000-GOMS3_-GOMS3_4_____-'spectral channel'_076E-000005___-'date'-C_
Number of segments: 36 body segments and Epi / Pro file
Temporal resolution: Every 30 minutes, starting from 00:00 UTC
Initial file format: wavelet compressed segments

Data is provided throught EUMETCast reception. For Legal Constraints check the 'Product Navigator' at EUMETSAT available at http://navigator.eumetsat.int
""")
   msg.pack()


#main()
#automatically setting of path when running this script from main menu - uncomment to make active   
gnc_path = os.environ["gnc_popup"]

#check path
print (gnc_path)
app = Tk()
app.title("Electro-L")
app.iconbitmap(gnc_path +'\pics\gnc.ico')

w = 650 # width for the Tk app
h = 210 # height for the Tk app

app.geometry('%dx%d' % (w, h))
app.resizable(0,0)

labelText1 = StringVar()
labelText1.set ("""Electro-L import routine   
Typical filename: H-000-GOMS3_-GOMS3_4_____-'spectral channel'_076E-000005___-'date'-C_""")

label1 = Label(app, textvariable=labelText1, anchor='w', justify = LEFT)
label1.pack(side = LEFT)
label1.place(x = 10, y = 5, width=650, height=35)

L1 = Label(app, text="Input Directory:", anchor='w')
L1.pack(side = LEFT)
L1.place(x = 10, y = 50, width=150, height=25)

inputDir = StringVar()
E1 = Entry(app, textvariable=inputDir, bd =2)
E1.pack()
E1.place(x = 165, y = 50, width=445, height=25)

button_dirin = Button(app, text="....", width=2, command = lambda: inputDir.set(askdirectory(initialdir = inputDir.get())))
button_dirin.pack()
button_dirin.place(x = 615, y = 49)

L2 = Label(app, text="Time Stamp (in UTC-time):", anchor='w')
L2.pack(side = LEFT)
L2.place(x = 10, y = 80, width=150, height=25)

inputFile = StringVar()
E2 = Entry(app, textvariable=inputFile, bd =2)
E2.pack()
E2.place(x = 165, y = 80, width=445, height=25)

#button_filein = Button(app, text="....", width=2, command = lambda: inputFile.set(askopenfilename(initialdir = inputDir.get()).split('/')[-1]))
#button_filein.pack()
#button_filein.place(x = 615, y = 79)

L3 = Label(app, text="Output Directory:", anchor='w')
L3.pack(side = LEFT)
L3.place(x = 10, y = 110, width=150, height=25)

outputDir = StringVar()
E3 = Entry(app, textvariable=outputDir, bd =2)
E3.pack()
E3.place(x = 165, y = 110, width=445, height=25)

button_dirout = Button(app, text="....", width=2, command = lambda: outputDir.set(askdirectory(initialdir = outputDir.get())))
button_dirout.pack()
button_dirout.place(x = 615, y = 109) 

L4 = Label(app, text="Output Filename:", anchor='w')
L4.pack(side = LEFT)
L4.place(x = 10, y = 140, width=150, height=25)

outputFile = StringVar()
E4 = Entry(app, textvariable=outputFile, bd =2)
E4.pack()
E4.place(x = 165, y = 140, width=445, height=25)

button0 = Button(app, text="Quit", width=15, command=app.destroy)
button0.pack()
button0.place(x = 525, y = 180)

button1 = Button(app, text="Execute", width=15, command=lambda: executeInThread(inputDir.get(), inputFile.get(), outputDir.get(), outputFile.get()))
button1.pack()
button1.place(x = 405, y = 180)

button2 = Button(app, text="About Electro-L", width=15, command=clickAbout)
button2.pack()
button2.place(x = 285, y = 180)

button3 = Button(app, text="Thumbnail", width=15, command=clickthumbn)
button3.pack()
button3.place(x = 165, y = 180)


style = Style()
style.theme_use('clam')
style.configure('blue.Horizontal.TProgressbar', foreground='blue', background='blue')

progress = Progressbar(app, style='blue.Horizontal.TProgressbar', orient=HORIZONTAL, length=100, mode='determinate')
progress.pack()
progress.place(x = 10, y = 180)


app.mainloop()

print("inputFile=" + inputFile.get())
print("outputDir=" + outputDir.get())
print("outputFile=" + outputFile.get())


