#Fengyun 2H Visible channel 1km resolution import routine
        
from tkinter import *
from tkinter.filedialog import askdirectory
from tkinter.filedialog import askopenfilename
from tkinter.ttk import Progressbar, Style
import shutil
import sys, os
import ilwis
import glob
import subprocess
from threading import Thread
import numpy as np
import gzip
import h5py

#passing parameters from sub menu

def execute(GNCDir, InFileName, WorkDir, OutFileName):
    if len(GNCDir) > 0 and len(InFileName) > 0 and len(WorkDir) > 0 and len(OutFileName) > 0:

        progress['value'] = 0
                
        time_stamp = InFileName

        os.chdir(WorkDir)
        ilwis.setWorkingCatalog(WorkDir)
        
        #filename = Z_SATE_C_BABJ_20250125230000_O_FY2H_FDI_VIS1KM_001_NOM.HDF.gz
        for fnames in glob.glob(os.path.join(GNCDir+'/Z_SATE_C_BABJ_'+str(time_stamp)+'00_O_FY2H_FDI_VIS1KM_001_NOM.HDF.gz')):
            shutil.copy(fnames, WorkDir)

        progress['value'] = 20        
            
        def extract_gz(file_path, output_path):
            with gzip.open(file_path, 'rb') as f_in:
                with open(output_path, 'wb') as f_out:
                    shutil.copyfileobj(f_in, f_out)
                    
        #Extracting file
        extract_gz(WorkDir+'/Z_SATE_C_BABJ_'+str(time_stamp)+'00_O_FY2H_FDI_VIS1KM_001_NOM.HDF.gz', WorkDir+'/FY_VIS1km.hdf')

        progress['value'] = 40   
 
        fn = WorkDir+ '/FY_VIS1km.hdf'

        # Open the HDF5 file in read mode
        with h5py.File(fn, 'r') as file:
            # Access the 'NOMChannelVIS1KM' dataset
            dataset = file['NOMChannelVIS1KM']
            # Convert the dataset to a NumPy array
            data_array = dataset[:]
        
        progress['value'] = 80
        
        grf = ilwis.GeoReference('code=georef:type=corners,csy=proj4:+proj=geos +a=6378137 +lon_0=79 +k_0=0.997800 +ellps=WGS84 +h=35785864 +rf=298.257223563, envelope=-5720000 5720000 5720000 -5720000, gridsize=9152 9152, cornerofcorners=yes')
        dfNum = ilwis.DataDefinition(ilwis.NumericDomain('code=value'), ilwis.NumericRange(0, 255, 0))
        rcNew = ilwis.RasterCoverage()
        rcNew.setSize(ilwis.Size(9152,9152,1))
        rcNew.setGeoReference(grf)
        rcNew.setDataDef(dfNum)
        
        rcNew.array2raster(data_array.flatten())
        
        rcNew = ilwis.do('mapcalc', 'iff(@1==0,?,@1)', rcNew) # remove background
        rcNew.store(OutFileName+'.mpr')

        progress['value'] = 100
        
        button0.config(state='normal')
        button1.config(state='normal')

#redundant files deleted in main menu script

def executeInThread(GNCDir, InFileName, WorkDir, OutFileName):
    button0.config(state='disabled')
    button1.config(state='disabled')
    thread = Thread(target = execute, args = (GNCDir, InFileName, WorkDir, OutFileName))
    thread.start()



# for command=doNothing
def doNothing():
   text="Do nothing button"
   return


# for command=clickthumbn
def clickthumbn():
   novi = Toplevel(app)
   novi.title ('Example of imported FY2H - VIS image (in ILWIS)')
   novi.iconbitmap(gnc_path +'\pics\gnc.ico')
   canvas = Canvas(novi, width = 400, height = 404)
   canvas.pack(expand = YES, fill = BOTH)
   gif1 = PhotoImage(file = gnc_path +'\pics\FY2H_Vis.gif')
   canvas.create_image(0, 0,image = gif1, anchor = NW)
   canvas.gif1 = gif1

# for command=clickAbout
def clickAbout(): 
   filewin = Toplevel(app)
   filewin.title ('About FY2H - VIS')
   filewin.iconbitmap(gnc_path +'\pics\gnc.ico')
   filewin.geometry("575x340")
   msg = Message(filewin, text="""FY2H VIS background:

Full disk image of the Visible channel from the Chinese geostationary satellite FengYun 2H, situated at 79 degree East. No data calibration can be performed as no information is provided in the meta-data. Quality of geometry is established by trial and eror and is as good as it gets! Spatial resolution of the visible channel is 1.25 KM.

File name convertion: Z_SATE_C_BABJ_20250125230000_O_FY2H_FDI_VIS1KM_001_NOM.HDF.gz
Number of segments: 1 body segment
Temporal Resolution: 1 hour
Initial file format: HDF5

Data is provided throught EUMETCast reception. For Legal Constraints check the 'Product Navigator' at EUMETSAT available at http://navigator.eumetsat.int
""")
   msg.pack()


#main()
#automatically setting of path when running this script from main menu - uncomment to make active   
gnc_path = os.environ["gnc_popup"]

#check path
print (gnc_path)
app = Tk()
app.title("Fengyun 2H Visible channel import routine")
app.iconbitmap(gnc_path +'\pics\gnc.ico')

w = 650 # width for the Tk app
h = 210 # height for the Tk app

app.geometry('%dx%d' % (w, h))
app.resizable(0,0)

labelText1 = StringVar()
labelText1.set ("""FY2H 
Typical filename: Z_SATE_C_BABJ_20250125230000_O_FY2H_FDI_VIS1KM_001_NOM.HDF.gz""")

label1 = Label(app, textvariable=labelText1, anchor='w', justify = LEFT)
label1.pack(side = LEFT)
label1.place(x = 10, y = 5, width=650, height=35)

L1 = Label(app, text="Input Directory:", anchor='w')
L1.pack(side = LEFT)
L1.place(x = 10, y = 50, width=150, height=25)

inputDir = StringVar()
E1 = Entry(app, textvariable=inputDir, bd =2)
E1.pack()
E1.place(x = 165, y = 50, width=445, height=25)

button_dirin = Button(app, text="....", width=2, command = lambda: inputDir.set(askdirectory(initialdir = inputDir.get())))
button_dirin.pack()
button_dirin.place(x = 615, y = 49)

L2 = Label(app, text="Time Stamp (in UTC-time):", anchor='w')
L2.pack(side = LEFT)
L2.place(x = 10, y = 80, width=150, height=25)

inputFile = StringVar()
E2 = Entry(app, textvariable=inputFile, bd =2)
E2.pack()
E2.place(x = 165, y = 80, width=445, height=25)

#button_filein = Button(app, text="....", width=2, command = lambda: inputFile.set(askopenfilename(initialdir = inputDir.get()).split('/')[-1]))
#button_filein.pack()
#button_filein.place(x = 615, y = 79)

L3 = Label(app, text="Output Directory:", anchor='w')
L3.pack(side = LEFT)
L3.place(x = 10, y = 110, width=150, height=25)

outputDir = StringVar()
E3 = Entry(app, textvariable=outputDir, bd =2)
E3.pack()
E3.place(x = 165, y = 110, width=445, height=25)

button_dirout = Button(app, text="....", width=2, command = lambda: outputDir.set(askdirectory(initialdir = outputDir.get())))
button_dirout.pack()
button_dirout.place(x = 615, y = 109) 

L4 = Label(app, text="Output Filename:", anchor='w')
L4.pack(side = LEFT)
L4.place(x = 10, y = 140, width=150, height=25)

outputFile = StringVar()
E4 = Entry(app, textvariable=outputFile, bd =2)
E4.pack()
E4.place(x = 165, y = 140, width=445, height=25)

button0 = Button(app, text="Quit", width=15, command=app.destroy)
button0.pack()
button0.place(x = 525, y = 180)

button1 = Button(app, text="Execute", width=15, command=lambda: executeInThread(inputDir.get(), inputFile.get(), outputDir.get(), outputFile.get()))
button1.pack()
button1.place(x = 405, y = 180)

button2 = Button(app, text="About FY2H", width=15, command=clickAbout)
button2.pack()
button2.place(x = 285, y = 180)

button3 = Button(app, text="Thumbnail", width=15, command=clickthumbn)
button3.pack()
button3.place(x = 165, y = 180)


style = Style()
style.theme_use('clam')
style.configure('blue.Horizontal.TProgressbar', foreground='blue', background='blue')

progress = Progressbar(app, style='blue.Horizontal.TProgressbar', orient=HORIZONTAL, length=100, mode='determinate')
progress.pack()
progress.place(x = 10, y = 180)


app.mainloop()

print("inputFile=" + inputFile.get())
print("outputDir=" + outputDir.get())
print("outputFile=" + outputFile.get())



