#GOES West (currently GOES 18) ABI
# 10 minutes files (16 in total) 
# requires start time - in 10 minutes intervals, e.g. yyyymmddhhmm, last mm  = 00, 10, 20, 30, 40, 50
# retrieves from date string doy number

from tkinter import *
from tkinter.filedialog import askdirectory
from tkinter.filedialog import askopenfilename
from tkinter.ttk import Progressbar, Style

import shutil
import sys, os 
import glob
import subprocess

import ilwis
import numpy as np

from satpy.scene import Scene
from satpy.resample import get_area_def
from satpy import find_files_and_readers

from datetime import datetime

from threading import Thread
import zipfile

#passing parameters from sub menu

def execute(GNCDir, InFileName, WorkDir, OutFileName):
    if len(GNCDir) > 0 and len(InFileName) > 0 and len(WorkDir) > 0 and len(OutFileName) > 0:

        progress['value'] = 0
        
        time_stamp = InFileName
        yyyy = int(time_stamp[0:4])
        mm = int(time_stamp[4:6])
        dd = int(time_stamp[6:8])
        tt = (time_stamp[8:11]) #only first 3 digits are used here!

        doy = datetime(yyyy, mm, dd).timetuple().tm_yday
        #print(doy)
        #print(tt)

        doys = "{:03d}".format(doy)
        #print(doys)

        os.chdir(WorkDir)
        ilwis.setWorkingCatalog(WorkDir)
        
        progress['value'] = 10
        
        for fnames in glob.glob(os.path.join(GNCDir+'/OR_ABI-L1b-RadF-M6C*_G18_s'+str(yyyy)+str(doys)+str(tt)+'*.nc')):
            shutil.copy(fnames, WorkDir)
            
        files = find_files_and_readers(base_dir=WorkDir, reader='abi_l1b')
        print(files)
        
        #read scene using satpy
        scn = Scene(filenames=files)
        print(scn.available_dataset_names())

        progress['value'] = 20
        
        #load values of spectral channels
        scn.load(["C01"])
        B1 = scn["C01"].values
            
        scn.load(["C02"])
        B2 = scn["C02"].values
        
        scn.load(["C03"])
        B3 = scn["C03"].values
        
        scn.load(["C04"])
        B4 = scn["C04"].values
        
        scn.load(["C05"])
        B5 = scn["C05"].values
            
        scn.load(["C06"])
        B6 = scn["C06"].values
        
        scn.load(["C07"])
        B7 = scn["C07"].values
        
        scn.load(["C08"])
        B8 = scn["C08"].values
        
        progress['value'] = 40
        
        scn.load(["C09"])
        B9 = scn["C09"].values
            
        scn.load(["C10"])
        B10 = scn["C10"].values
        
        scn.load(["C11"])
        B11 = scn["C11"].values
        
        scn.load(["C12"])
        B12 = scn["C12"].values
        
        scn.load(["C13"])
        B13 = scn["C13"].values
        
        progress['value'] = 60
            
        scn.load(["C14"])
        B14 = scn["C14"].values
        
        scn.load(["C15"])
        B15 = scn["C15"].values
        
        scn.load(["C16"])
        B16 = scn["C16"].values
        
        data = np.array((B1, B2, B3, B4, B5, B6, B7, B8, B9, B10, B11, B12, B13, B14, B15, B16))
        

        #export to ILWIS format -create an empty raster
        grf = ilwis.GeoReference('code=georef:type=corners,csy=proj4:+proj=geos +ellps=GRS80 +lon_0=-137 +sweep=x +h=35786023,envelope=-5435395.8894 5435395.8894 5434393.8807 -5434393.8807, gridsize=5424 5424, cornerofcorners=yes')
        dfNum = ilwis.DataDefinition(ilwis.NumericDomain('code=value'), ilwis.NumericRange(-1000.0, 1000.0, 0.00001))
        rcNew = ilwis.RasterCoverage()
        rcNew.setSize(ilwis.Size(5424,5424,16))
        rcNew.setGeoReference(grf)
        rcNew.setDataDef(dfNum)
        
        #write data to raster
        #rcNew.array2raster(data)
        rcNew.array2raster(data.flatten())
        
        progress['value'] = 80
            
        #correct background and save the results to disk as an ILWIS mpr file
        rcNew = ilwis.do('mapcalc', 'iff(@1>=-100,@1,?)', rcNew) # remove (satpy) nodata
        rcNew.store(OutFileName+'.mpl')  
      
        #change in coordinate system file *.csy the respective satellite projection from geostationary to geostationary goes
        #List of the file names to be changed
        files = [
            WorkDir+'/'+OutFileName+'_mpl_band_0.csy',
            WorkDir+'/'+OutFileName+'_mpl_band_1.csy',
            WorkDir+'/'+OutFileName+'_mpl_band_2.csy',
            WorkDir+'/'+OutFileName+'_mpl_band_3.csy',
            WorkDir+'/'+OutFileName+'_mpl_band_4.csy',
            WorkDir+'/'+OutFileName+'_mpl_band_5.csy',
            WorkDir+'/'+OutFileName+'_mpl_band_6.csy',
            WorkDir+'/'+OutFileName+'_mpl_band_7.csy',
            WorkDir+'/'+OutFileName+'_mpl_band_8.csy',
            WorkDir+'/'+OutFileName+'_mpl_band_9.csy',
            WorkDir+'/'+OutFileName+'_mpl_band_10.csy',
            WorkDir+'/'+OutFileName+'_mpl_band_11.csy',
            WorkDir+'/'+OutFileName+'_mpl_band_12.csy',
            WorkDir+'/'+OutFileName+'_mpl_band_13.csy',
            WorkDir+'/'+OutFileName+'_mpl_band_14.csy',
            WorkDir+'/'+OutFileName+'_mpl_band_15.csy',
            WorkDir+'/'+OutFileName+'_mpl_band_16.csy',
            WorkDir+'/'+OutFileName+'.csy'
        ]

        # Original line to search for and the new line to replace it
        old_line = 'Projection=GeoStationary Satellite'
        new_line = 'Projection=GeoStationary Satellite Goes'

        # Loop through each file
        for filename in files:
            try:
                # Open the file in read mode and read all lines
                with open(filename, 'r') as file:
                    lines = file.readlines()
        
                # Modify the lines where necessary
                with open(filename, 'w') as file:
                    for line in lines:
                        if old_line in line:
                            # Replace the old line with the new one
                            line = line.replace(old_line, new_line)
                        file.write(line)
        
                print(f'Processed file: {filename}')
    
            except FileNotFoundError:
                print(f'File not found: {filename}')
            except Exception as e:
                print(f'Error processing {filename}: {e}')        

           
        progress['value'] = 100
        button0.config(state='normal')
        button1.config(state='normal')

#redundant files deleted in main menu script

def executeInThread(GNCDir, InFileName, WorkDir, OutFileName):
    button0.config(state='disabled')
    button1.config(state='disabled')
    thread = Thread(target = execute, args = (GNCDir, InFileName, WorkDir, OutFileName))
    thread.start()



# for command=doNothing
def doNothing():
   text="Do nothing button"
   return


# for command=clickthumbn
def clickthumbn():
   novi = Toplevel(app)
   novi.title ('Example of imported GOES West ABI image (in ILWIS)')
   novi.iconbitmap(gnc_path +'\pics\gnc.ico')
   canvas = Canvas(novi, width = 400, height = 403)
   canvas.pack(expand = YES, fill = BOTH)
   gif1 = PhotoImage(file = gnc_path +'\pics\g18_abi.gif')
   canvas.create_image(0, 0,image = gif1, anchor = NW)
   canvas.gif1 = gif1

# for command=clickAbout
def clickAbout(): 
   filewin = Toplevel(app)
   filewin.title ('About GOES ABI')
   filewin.iconbitmap(gnc_path +'\pics\gnc.ico')
   filewin.geometry("575x340")
   msg = Message(filewin, text="""GOES ABI background:

the full disk L1b data in the visible, near infrared, and infrared spectral region from the Advanced Baseline Imager (ABI), onboard NOAA's GOES-18 satellite.

File name convertion: OR_ABI-L1b-RadF-M6C*_G18_s*.nc
Number of segments: 16 files are retrieved, one for each spectral channel
Temporal resolution: Every 10 minutes, starting from 00:00 UTC
Initial file format: NetCDF

Data is provided throught EUMETCast reception. For Legal Constraints check the 'Product Navigator' at EUMETSAT available at http://navigator.eumetsat.int
""")
   msg.pack()


#main()
#automatically setting of path when running this script from main menu - uncomment to make active   
gnc_path = os.environ["gnc_popup"]

#check path
print (gnc_path)
app = Tk()
app.title("GOES WEST ABI import routine")
app.iconbitmap(gnc_path +'\pics\gnc.ico')

w = 650 # width for the Tk app
h = 210 # height for the Tk app

app.geometry('%dx%d' % (w, h))
app.resizable(0,0)

labelText1 = StringVar()
labelText1.set ("""GOES West ABI 
Typical filename: OR_ABI-L1b-RadF-M6C*_G18_s*.nc""")

label1 = Label(app, textvariable=labelText1, anchor='w', justify = LEFT)
label1.pack(side = LEFT)
label1.place(x = 10, y = 5, width=650, height=35)

L1 = Label(app, text="Input Directory:", anchor='w')
L1.pack(side = LEFT)
L1.place(x = 10, y = 50, width=150, height=25)

inputDir = StringVar()
E1 = Entry(app, textvariable=inputDir, bd =2)
E1.pack()
E1.place(x = 165, y = 50, width=445, height=25)

button_dirin = Button(app, text="....", width=2, command = lambda: inputDir.set(askdirectory(initialdir = inputDir.get())))
button_dirin.pack()
button_dirin.place(x = 615, y = 49)

L2 = Label(app, text="Time (yyyymmddhhmm):", anchor='w')
L2.pack(side = LEFT)
L2.place(x = 10, y = 80, width=150, height=25)

inputFile = StringVar()
E2 = Entry(app, textvariable=inputFile, bd =2)
E2.pack()
E2.place(x = 165, y = 80, width=445, height=25)

#button_filein = Button(app, text="....", width=2, command = lambda: inputFile.set(askopenfilename(initialdir = inputDir.get()).split('/')[-1]))
#button_filein.pack()
#button_filein.place(x = 615, y = 79)

L3 = Label(app, text="Output Directory:", anchor='w')
L3.pack(side = LEFT)
L3.place(x = 10, y = 110, width=150, height=25)

outputDir = StringVar()
E3 = Entry(app, textvariable=outputDir, bd =2)
E3.pack()
E3.place(x = 165, y = 110, width=445, height=25)

button_dirout = Button(app, text="....", width=2, command = lambda: outputDir.set(askdirectory(initialdir = outputDir.get())))
button_dirout.pack()
button_dirout.place(x = 615, y = 109) 

L4 = Label(app, text="Output Filename prefix:", anchor='w')
L4.pack(side = LEFT)
L4.place(x = 10, y = 140, width=150, height=25)

outputFile = StringVar()
E4 = Entry(app, textvariable=outputFile, bd =2)
E4.pack()
E4.place(x = 165, y = 140, width=445, height=25)

button0 = Button(app, text="Quit", width=15, command=app.destroy)
button0.pack()
button0.place(x = 525, y = 180)

button1 = Button(app, text="Execute", width=15, command=lambda: executeInThread(inputDir.get(), inputFile.get(), outputDir.get(), outputFile.get()))
button1.pack()
button1.place(x = 405, y = 180)

button2 = Button(app, text="About GOES-ABI", width=15, command=clickAbout)
button2.pack()
button2.place(x = 285, y = 180)

button3 = Button(app, text="Thumbnail", width=15, command=clickthumbn)
button3.pack()
button3.place(x = 165, y = 180)


style = Style()
style.theme_use('clam')
style.configure('blue.Horizontal.TProgressbar', foreground='blue', background='blue')

progress = Progressbar(app, style='blue.Horizontal.TProgressbar', orient=HORIZONTAL, length=100, mode='determinate')
progress.pack()
progress.place(x = 10, y = 180)


app.mainloop()

print("inputFile=" + inputFile.get())
print("outputDir=" + outputDir.get())
print("outputFile=" + outputFile.get())



