#HSAF H14 SM DAS import routine
from tkinter import *
from tkinter.filedialog import askdirectory
from tkinter.filedialog import askopenfilename
from tkinter.ttk import Progressbar, Style
import os
import sys
import shutil
import glob
import subprocess

import ilwis
import netCDF4 as nc
from threading import Thread
import subprocess
import numpy as np
import bz2


#passing parameters from sub menu

def execute(GNCDir, InFileName, WorkDir, OutFileName):
    if len(GNCDir) > 0 and len(InFileName) > 0 and len(WorkDir) > 0 and len(OutFileName) > 0:

        progress['value'] = 0
        time_stamp_in = InFileName
        timeA = time_stamp_in[:8]
        timeB = time_stamp_in[8:12]
        
        progress['value'] = 10
        
        os.chdir(WorkDir)
        ilwis.setWorkingCatalog(WorkDir)
        
        #filename expected =  S-HSAF-h14_20250407_0000.grib.bz2
        for filename in glob.glob(os.path.join(GNCDir+'/S-HSAF-h14_'+timeA+'_'+timeB+'.grib.bz2')):
            shutil.copy(filename, WorkDir)
        
        progress['value'] = 20

        for fnames in glob.glob(os.path.join(WorkDir+'/S-HSAF-h14_'+ timeA +'_'+timeB+'.grib.bz2')):
            file_in = fnames

        fname = file_in.rsplit('.',1)[0] #get file name without bz2 extension
        with bz2.open(file_in,"rb") as f_in, open(fname,"wb") as f_out:
            f_out.write(f_in.read()) 
            
        #redirect popup MSDOS window when executing the sub process
        si = subprocess.STARTUPINFO()
        si.dwFlags |= subprocess.STARTF_USESHOWWINDOW
            
        decode = subprocess.run([gnc_path+'/extern/Conv/convsh191.exe',gnc_path+'/extern/Conv/smdas22nc.tcl', '-i', fname, '-o', WorkDir+'/grib_out.nc'], startupinfo=si)    
            
        ds = nc.Dataset(WorkDir+'/grib_out.nc')           

        progress['value'] =40
        
        #load H14 data 
        data = ds['DZDT'] #for DZDT, ABSV, ABSD, RELV
        data_new = data[0][0]

        data_east = data_new[0:800,0:800]
        data_west = data_new[0:800,800:1600]
        
        dzdt = np.hstack((data_west, data_east))
        max_value = np.nanmax(dzdt)
        dzdt[dzdt == max_value] = -999
        
        data = ds['ABSV'] #for DZDT, ABSV, ABSD, RELV
        data_new = data[0][0]

        data_east = data_new[0:800,0:800]
        data_west = data_new[0:800,800:1600]
    
        absv = np.hstack((data_west, data_east))
        max_value = np.nanmax(absv)
        absv[absv == max_value] = -999
        
        data = ds['ABSD'] #for DZDT, ABSV, ABSD, RELV
        data_new = data[0][0]

        data_east = data_new[0:800,0:800]
        data_west = data_new[0:800,800:1600]

        absd = np.hstack((data_west, data_east))
        max_value = np.nanmax(absd)
        absd[absd == max_value] = -999
        
        data = ds['RELV'] #for DZDT, ABSV, ABSD, RELV
        data_new = data[0][0]

        data_east = data_new[0:800,0:800]
        data_west = data_new[0:800,800:1600]

        relv = np.hstack((data_west, data_east))
        max_value = np.nanmax(relv)
        relv[relv == max_value] = -999
               
        progress['value'] = 60
        
        rcAll = np.array([dzdt,absv,absd,relv])
        
        #create output map in ILWIS format
        grf = ilwis.GeoReference('code=georef:type=corners, csy=epsg:4326, envelope=-180 90 180 -90, gridsize=1600 800, cornerofcorners=yes')
        dfNum = ilwis.DataDefinition(ilwis.NumericDomain('code=value'), ilwis.NumericRange(0, 1, 0))
        rc = ilwis.RasterCoverage()
        rc.setSize(ilwis.Size(1600,800,4))
        rc.setGeoReference(grf)
        rc.setDataDef(dfNum)

        rc_4l = rcAll.flatten()
        rc.array2raster(rc_4l)
        
        progress['value'] = 80
        
        rc_cor = ilwis.do('mapcalc','iff(@1<=-999,?,@1)',rc)
        rc_cor.store(OutFileName+'.mpl')

        #Other redundant files are deleted in main menu script
        progress['value'] = 100
        
        button0.config(state='normal')
        button1.config(state='normal')

def executeInThread(GNCDir, InFileName, WorkDir, OutFileName):
    button0.config(state='disabled')
    button1.config(state='disabled')
    thread = Thread(target = execute, args = (GNCDir, InFileName, WorkDir, OutFileName))
    thread.start()



# for command=doNothing
def doNothing():
   text="Do nothing button"
   return


# for command=clickthumbn
def clickthumbn():
   novi = Toplevel(app)
   novi.title ('Example of visualized HSAF - H14 image (in ILWIS)')
   novi.iconbitmap(gnc_path +'\pics\gnc.ico')
   canvas = Canvas(novi, width = 838, height = 400)
   canvas.pack(expand = YES, fill = BOTH)
   gif1 = PhotoImage(file = gnc_path +'\pics\HSAF_H14.gif')
   canvas.create_image(0, 0,image = gif1, anchor = NW)
   canvas.gif1 = gif1

# for command=clickAbout
def clickAbout(): 
   filewin = Toplevel(app)
   filewin.title ('About H14')
   filewin.iconbitmap(gnc_path +'\pics\gnc.ico')
   filewin.geometry("575x340")
   msg = Message(filewin, text="""HSAF H14B background:

ASCAT Root Zone Soil Moisture Profile Index 25 km resolution in NRT - Metop. The surface observation from ASCAT is propagated towards the roots region down to 2.89 m below surface, providing estimates for 4 layers (thicknesses 0.07, 0.21, 0.72 and 1.89 m). The ECMWF model generates soil moisture profile information according to the Hydrology Tiled ECMWF Scheme for Surface Exchanges over Land (HTESSEL). SM-DAS-2 is available at a 24-hour time step, with a global daily coverage at 00:00 UTC. SM-DAS-2 is produced in a continuous way in order to ensure the time series consistency of the product (and also to provide values when there is no satellite data, from the model propagation). The SM-DAS-2 product is the first global product of consistent surface and root zone soil moisture available NRT for the NWP, climate and hydrological communities. Resolution: Horizontal: 25 km (T799) Vertical (below surface): 4 layers: 0-7 cm, 7-28 cm, 28-100 cm, 100-289 cm 

File name convertion: S-HSAF-h14_20250409_0000.grib.bz2
Number of segments: 1 file
Temporal resolution: 1 per day, starting from 00:00 UTC
Initial file format: grib1 format

Data is provided throught EUMETCast reception. For Legal Constraints check the 'Product Navigator' at EUMETSAT available at http://navigator.eumetsat.int
""")
   msg.pack()


#main()
#automatically setting of path when running this script from main menu - uncomment to make active   
gnc_path = os.environ["gnc_popup"]

#check path
print (gnc_path)
app = Tk()
app.title("HSAF H14B")
app.iconbitmap(gnc_path +'\pics\gnc.ico')

w = 650 # width for the Tk app
h = 210 # height for the Tk app

app.geometry('%dx%d' % (w, h))
app.resizable(0,0)

labelText1 = StringVar()
labelText1.set ("""HSAF H14B import routine   
Typical filename: S-HSAF-h14_20250409_0000.grib.bz2""")

label1 = Label(app, textvariable=labelText1, anchor='w', justify = LEFT)
label1.pack(side = LEFT)
label1.place(x = 10, y = 5, width=650, height=35)

L1 = Label(app, text="Input Directory:", anchor='w')
L1.pack(side = LEFT)
L1.place(x = 10, y = 50, width=150, height=25)

inputDir = StringVar()
E1 = Entry(app, textvariable=inputDir, bd =2)
E1.pack()
E1.place(x = 165, y = 50, width=445, height=25)

button_dirin = Button(app, text="....", width=2, command = lambda: inputDir.set(askdirectory(initialdir = inputDir.get())))
button_dirin.pack()
button_dirin.place(x = 615, y = 49)

L2 = Label(app, text="Time Stamp (in UTC-time):", anchor='w')
L2.pack(side = LEFT)
L2.place(x = 10, y = 80, width=150, height=25)

inputFile = StringVar()
E2 = Entry(app, textvariable=inputFile, bd =2)
E2.pack()
E2.place(x = 165, y = 80, width=445, height=25)

#button_filein = Button(app, text="....", width=2, command = lambda: inputFile.set(askopenfilename(initialdir = inputDir.get()).split('/')[-1]))
#button_filein.pack()
#button_filein.place(x = 615, y = 79)

L3 = Label(app, text="Output Directory:", anchor='w')
L3.pack(side = LEFT)
L3.place(x = 10, y = 110, width=150, height=25)

outputDir = StringVar()
E3 = Entry(app, textvariable=outputDir, bd =2)
E3.pack()
E3.place(x = 165, y = 110, width=445, height=25)

button_dirout = Button(app, text="....", width=2, command = lambda: outputDir.set(askdirectory(initialdir = outputDir.get())))
button_dirout.pack()
button_dirout.place(x = 615, y = 109) 

L4 = Label(app, text="Output Filename:", anchor='w')
L4.pack(side = LEFT)
L4.place(x = 10, y = 140, width=150, height=25)

outputFile = StringVar()
E4 = Entry(app, textvariable=outputFile, bd =2)
E4.pack()
E4.place(x = 165, y = 140, width=445, height=25)

button0 = Button(app, text="Quit", width=15, command=app.destroy)
button0.pack()
button0.place(x = 525, y = 180)

button1 = Button(app, text="Execute", width=15, command=lambda: executeInThread(inputDir.get(), inputFile.get(), outputDir.get(), outputFile.get()))
button1.pack()
button1.place(x = 405, y = 180)

button2 = Button(app, text="About H14", width=15, command=clickAbout)
button2.pack()
button2.place(x = 285, y = 180)

button3 = Button(app, text="Thumbnail", width=15, command=clickthumbn)
button3.pack()
button3.place(x = 165, y = 180)


style = Style()
style.theme_use('clam')
style.configure('blue.Horizontal.TProgressbar', foreground='blue', background='blue')

progress = Progressbar(app, style='blue.Horizontal.TProgressbar', orient=HORIZONTAL, length=100, mode='determinate')
progress.pack()
progress.place(x = 10, y = 180)


app.mainloop()

print("inputFile=" + inputFile.get())
print("outputDir=" + outputDir.get())
print("outputFile=" + outputFile.get())


