#Himawari import all channels
#channel sequence stored according to: https://www.data.jma.go.jp/mscweb/en/himawari89/space_segment/spsg_ahi.html
# 30 minutes files (10 per spectral channel, 160 in total) 
# requires start time - in 30 minutes intervals, e.g. yyyymmddhhmm, last mm  = 00, 30


from tkinter import *
from tkinter.filedialog import askdirectory
from tkinter.filedialog import askopenfilename
from tkinter.ttk import Progressbar, Style

import shutil
import sys, os 
import glob
import subprocess

import ilwis
import numpy as np

from satpy.scene import Scene
from satpy.resample import get_area_def
from satpy import find_files_and_readers

#from datetime import datetime
from threading import Thread


#passing parameters from sub menu

def execute(GNCDir, InFileName, WorkDir, OutFileName):
    if len(GNCDir) > 0 and len(InFileName) > 0 and len(WorkDir) > 0 and len(OutFileName) > 0:

        progress['value'] = 0
        
        time_stamp = InFileName
        yyyy = (time_stamp[0:4])
        mm = (time_stamp[4:6])
        dd = (time_stamp[6:8])
        tt = (time_stamp[8:11]) #only first 3 digits are used here!
        
        print(GNCDir+'/IMG_DK01B01_'+str(yyyy)+str(mm)+str(dd)+str(tt)+'*.bz2')
        os.chdir(WorkDir)
        ilwis.setWorkingCatalog(WorkDir)
        
        progress['value'] = 10
        
        for fnames in glob.glob(os.path.join(GNCDir+'/IMG_DK01B*_'+str(yyyy)+str(mm)+str(dd)+str(tt)+'*.bz2')):
            shutil.copy(fnames, WorkDir)
            
        for fnames in glob.glob(os.path.join(GNCDir+'/IMG_DK01IR*_'+str(yyyy)+str(mm)+str(dd)+str(tt)+'*.bz2')):
            shutil.copy(fnames, WorkDir)
            
        for fnames in glob.glob(os.path.join(GNCDir+'/IMG_DK01VIS_'+str(yyyy)+str(mm)+str(dd)+str(tt)+'*.bz2')):
            shutil.copy(fnames, WorkDir)    
            
        #files = find_files_and_readers(base_dir=WorkDir, reader='ahi_hrit')
        #print(files)
        
        progress['value'] = 20
        
        #read scene using satpy - B01
        fnames = glob.glob(WorkDir+'/IMG_DK01B01_'+str(yyyy)+str(mm)+str(dd)+str(tt)+'*.bz2')
        scn = Scene(reader='ahi_hrit',  filenames=fnames)
        scn.load(['B01'])
        B01_values = scn['B01'].values
        
        #read scene using satpy - B02        
        fnames = glob.glob(WorkDir+'/IMG_DK01B02_'+str(yyyy)+str(mm)+str(dd)+str(tt)+'*.bz2')
        scn = Scene(reader='ahi_hrit',  filenames=fnames)
        scn.load(['B02'])
        B02_values = scn['B02'].values
        
        
        #read scene using satpy - VIS - B03        
        fnames = glob.glob(WorkDir+'/IMG_DK01VIS_'+str(yyyy)+str(mm)+str(dd)+str(tt)+'*.bz2')
        scn = Scene(reader='ahi_hrit',  filenames=fnames)
        scn.load(['B03'])
        B03_values = scn['B03'].values
        
        #read scene using satpy - B04
        fnames = glob.glob(WorkDir+'/IMG_DK01B04_'+str(yyyy)+str(mm)+str(dd)+str(tt)+'*.bz2')
        scn = Scene(reader='ahi_hrit',  filenames=fnames)
        scn.load(['B04'])
        B04_values = scn['B04'].values
        
        #read scene using satpy - B05        
        fnames = glob.glob(WorkDir+'/IMG_DK01B05_'+str(yyyy)+str(mm)+str(dd)+str(tt)+'*.bz2')
        scn = Scene(reader='ahi_hrit',  filenames=fnames)
        scn.load(['B05'])
        B05_values = scn['B05'].values
        
        #read scene using satpy - B06        
        fnames = glob.glob(WorkDir+'/IMG_DK01B06_'+str(yyyy)+str(mm)+str(dd)+str(tt)+'*.bz2')
        scn = Scene(reader='ahi_hrit',  filenames=fnames)
        scn.load(['B06'])
        B06_values = scn['B06'].values
        
        progress['value'] = 40
        
        #read scene using satpy - IR4 - B07
        fnames = glob.glob(WorkDir+'/IMG_DK01IR4_'+str(yyyy)+str(mm)+str(dd)+str(tt)+'*.bz2')
        scn = Scene(reader='ahi_hrit',  filenames=fnames)
        scn.load(['B07'])
        B07_values = scn['B07'].values
        
        #read scene using satpy - IR3 - B08        
        fnames = glob.glob(WorkDir+'/IMG_DK01IR3_'+str(yyyy)+str(mm)+str(dd)+str(tt)+'*.bz2')
        scn = Scene(reader='ahi_hrit',  filenames=fnames)
        scn.load(['B08'])
        B08_values = scn['B08'].values
        
        #read scene using satpy - B09        
        fnames = glob.glob(WorkDir+'/IMG_DK01B09_'+str(yyyy)+str(mm)+str(dd)+str(tt)+'*.bz2')
        scn = Scene(reader='ahi_hrit',  filenames=fnames)
        scn.load(['B09'])
        B09_values = scn['B09'].values

        #read scene using satpy - B10
        fnames = glob.glob(WorkDir+'/IMG_DK01B10_'+str(yyyy)+str(mm)+str(dd)+str(tt)+'*.bz2')
        scn = Scene(reader='ahi_hrit',  filenames=fnames)
        scn.load(['B10'])
        B10_values = scn['B10'].values
        
        #read scene using satpy - B11        
        fnames = glob.glob(WorkDir+'/IMG_DK01B11_'+str(yyyy)+str(mm)+str(dd)+str(tt)+'*.bz2')
        scn = Scene(reader='ahi_hrit',  filenames=fnames)
        scn.load(['B11'])
        B11_values = scn['B11'].values
        
        #read scene using satpy - B12        
        fnames = glob.glob(WorkDir+'/IMG_DK01B12_'+str(yyyy)+str(mm)+str(dd)+str(tt)+'*.bz2')
        scn = Scene(reader='ahi_hrit',  filenames=fnames)
        scn.load(['B12'])
        B12_values = scn['B12'].values
        
        #read scene using satpy - IR1 - B13
        fnames = glob.glob(WorkDir+'/IMG_DK01IR1_'+str(yyyy)+str(mm)+str(dd)+str(tt)+'*.bz2')
        scn = Scene(reader='ahi_hrit',  filenames=fnames)
        scn.load(['B13'])
        B13_values = scn['B13'].values
        
        #read scene using satpy - B14        
        fnames = glob.glob(WorkDir+'/IMG_DK01B14_'+str(yyyy)+str(mm)+str(dd)+str(tt)+'*.bz2')
        scn = Scene(reader='ahi_hrit',  filenames=fnames)
        scn.load(['B14'])
        B14_values = scn['B14'].values
        
        #read scene using satpy - IR2 - B15
        fnames = glob.glob(WorkDir+'/IMG_DK01IR2_'+str(yyyy)+str(mm)+str(dd)+str(tt)+'*.bz2')
        scn = Scene(reader='ahi_hrit',  filenames=fnames)
        scn.load(['B15'])
        B15_values = scn['B15'].values

        #read scene using satpy - B16        
        fnames = glob.glob(WorkDir+'/IMG_DK01B16_'+str(yyyy)+str(mm)+str(dd)+str(tt)+'*.bz2')
        scn = Scene(reader='ahi_hrit',  filenames=fnames)
        scn.load(['B16'])
        B16_values = scn['B16'].values        
         
        progress['value'] = 60
        
        data = np.array((B01_values, B02_values, B03_values, B04_values, B05_values, B06_values, B07_values, B08_values, B09_values, B10_values, B11_values, B12_values, B13_values, B14_values, B15_values, B16_values))
        
        grf = ilwis.GeoReference('code=georef:type=corners,csy=proj4:+proj=geos +lon_0=140.7 +h=35785831 +a=6378196 +rf=295.488065897001,envelope=-5498994.9839 5500994.9821 5500994.9821 -5498994.9839,gridsize=5500 5500,cornerofcorners=yes')
        dfNum = ilwis.DataDefinition(ilwis.NumericDomain('code=value'), ilwis.NumericRange(-0.1, 255.0, 0.01))
        rcNew = ilwis.RasterCoverage()
        rcNew.setSize(ilwis.Size(5500,5500,16))
        rcNew.setGeoReference(grf)
        rcNew.setDataDef(dfNum)
        
        rcNew.array2raster(data.flatten())
        
        progress['value'] = 80
        
        rcNew = ilwis.do('mapcalc', 'iff(@1>=-0.1,@1,?)', rcNew) # remove satpy nodata
        
        #save the results to disk as an ILWIS mpl file
        rcNew.store(OutFileName+'.mpl')   
        
        progress['value'] = 100
        button0.config(state='normal')
        button1.config(state='normal')

#redundant files deleted in main menu script

def executeInThread(GNCDir, InFileName, WorkDir, OutFileName):
    button0.config(state='disabled')
    button1.config(state='disabled')
    thread = Thread(target = execute, args = (GNCDir, InFileName, WorkDir, OutFileName))
    thread.start()



# for command=doNothing
def doNothing():
   text="Do nothing button"
   return


# for command=clickthumbn
def clickthumbn():
   novi = Toplevel(app)
   novi.title ('Example of imported Himawari FC image (in ILWIS)')
   novi.iconbitmap(gnc_path +'\pics\gnc.ico')
   canvas = Canvas(novi, width = 400, height = 400)
   canvas.pack(expand = YES, fill = BOTH)
   gif1 = PhotoImage(file = gnc_path +'\pics\Himawari_all_FC.gif')
   canvas.create_image(0, 0,image = gif1, anchor = NW)
   canvas.gif1 = gif1

# for command=clickAbout
def clickAbout(): 
   filewin = Toplevel(app)
   filewin.title ('About Himawari')
   filewin.iconbitmap(gnc_path +'\pics\gnc.ico')
   filewin.geometry("575x340")
   msg = Message(filewin, text="""Himawari background:

Level 1 data from the Advanced Himawari Imager (AHI). The data are from 16 channels at 2 km resolution, see: https://www.data.jma.go.jp/mscweb/en/himawari89/space_segment/spsg_ahi.html. Band assignment kept when band number is provided, else channels are renamed - Vis=B03, IR4=B07, IR3=B08, IR1=B13 and IR2=B15 respectively.

File name convertion: IMG_DK01<band>_<date>_<segment>.bz2
Number of segments: 160 segments are retrieved,10 for each spectral channel
Temporal resolution: Every 30 minutes, starting from 00:00 UTC
Initial file format: Himawari Standard Format (HSD) and converted to HRIT format at EUMETSAT

Data is provided throught EUMETCast reception. For Legal Constraints check the 'Product Navigator' at EUMETSAT available at http://navigator.eumetsat.int
""")
   msg.pack()


#main()
#automatically setting of path when running this script from main menu - uncomment to make active   
gnc_path = os.environ["gnc_popup"]

#check path
print (gnc_path)
app = Tk()
app.title("Himawari all spectral channels import routine")
app.iconbitmap(gnc_path +'\pics\gnc.ico')

w = 650 # width for the Tk app
h = 210 # height for the Tk app

app.geometry('%dx%d' % (w, h))
app.resizable(0,0)

labelText1 = StringVar()
labelText1.set ("""Advanced Himawari Imager all spectral channels import
Typical filename: IMG_DK01<band>_<date>_<segment>.bz2""")

label1 = Label(app, textvariable=labelText1, anchor='w', justify = LEFT)
label1.pack(side = LEFT)
label1.place(x = 10, y = 5, width=650, height=35)

L1 = Label(app, text="Input Directory:", anchor='w')
L1.pack(side = LEFT)
L1.place(x = 10, y = 50, width=150, height=25)

inputDir = StringVar()
E1 = Entry(app, textvariable=inputDir, bd =2)
E1.pack()
E1.place(x = 165, y = 50, width=445, height=25)

button_dirin = Button(app, text="....", width=2, command = lambda: inputDir.set(askdirectory(initialdir = inputDir.get())))
button_dirin.pack()
button_dirin.place(x = 615, y = 49)

L2 = Label(app, text="Time (yyyymmddhhmm):", anchor='w')
L2.pack(side = LEFT)
L2.place(x = 10, y = 80, width=150, height=25)

inputFile = StringVar()
E2 = Entry(app, textvariable=inputFile, bd =2)
E2.pack()
E2.place(x = 165, y = 80, width=445, height=25)

#button_filein = Button(app, text="....", width=2, command = lambda: inputFile.set(askopenfilename(initialdir = inputDir.get()).split('/')[-1]))
#button_filein.pack()
#button_filein.place(x = 615, y = 79)

L3 = Label(app, text="Output Directory:", anchor='w')
L3.pack(side = LEFT)
L3.place(x = 10, y = 110, width=150, height=25)

outputDir = StringVar()
E3 = Entry(app, textvariable=outputDir, bd =2)
E3.pack()
E3.place(x = 165, y = 110, width=445, height=25)

button_dirout = Button(app, text="....", width=2, command = lambda: outputDir.set(askdirectory(initialdir = outputDir.get())))
button_dirout.pack()
button_dirout.place(x = 615, y = 109) 

L4 = Label(app, text="Output Filename prefix:", anchor='w')
L4.pack(side = LEFT)
L4.place(x = 10, y = 140, width=150, height=25)

outputFile = StringVar()
E4 = Entry(app, textvariable=outputFile, bd =2)
E4.pack()
E4.place(x = 165, y = 140, width=445, height=25)

button0 = Button(app, text="Quit", width=15, command=app.destroy)
button0.pack()
button0.place(x = 525, y = 180)

button1 = Button(app, text="Execute", width=15, command=lambda: executeInThread(inputDir.get(), inputFile.get(), outputDir.get(), outputFile.get()))
button1.pack()
button1.place(x = 405, y = 180)

button2 = Button(app, text="About Himawari", width=15, command=clickAbout)
button2.pack()
button2.place(x = 285, y = 180)

button3 = Button(app, text="Thumbnail", width=15, command=clickthumbn)
button3.pack()
button3.place(x = 165, y = 180)


style = Style()
style.theme_use('clam')
style.configure('blue.Horizontal.TProgressbar', foreground='blue', background='blue')

progress = Progressbar(app, style='blue.Horizontal.TProgressbar', orient=HORIZONTAL, length=100, mode='determinate')
progress.pack()
progress.place(x = 10, y = 180)


app.mainloop()

print("inputFile=" + inputFile.get())
print("outputDir=" + outputDir.get())
print("outputFile=" + outputFile.get())



