#Meteosat Third Generation Lightning Imager Level 2 Lightning Flashes (LFL)
# 10 minutes files (60 in total) are integrated and exported to shape file / CSV 
# requires start time - in 10 minutes intervals, e.g. yyyymmddhhmm, last mm  = 00, 10, 20, 30, 40, 50

from tkinter import *
from tkinter.filedialog import askdirectory
from tkinter.filedialog import askopenfilename
from tkinter.ttk import Progressbar, Style

import shutil
import sys, os 
import glob
import subprocess
import datetime
import fnmatch
import time
import pandas as pd

import geopandas as gpd
from shapely.geometry import Point

import netCDF4
import numpy as np

from threading import Thread
import zipfile

#passing parameters from sub menu

def execute(GNCDir, InFileName, WorkDir, OutFileName):
    if len(GNCDir) > 0 and len(InFileName) > 0 and len(WorkDir) > 0 and len(OutFileName) > 0:

        progress['value'] = 0
        
        time_stamp = InFileName
        year = time_stamp[0:4]
        month = time_stamp[4:6]
        day = time_stamp[6:8]
        hour = time_stamp[8:10]
        minute = time_stamp[10:12]
        timehrUTC = time_stamp[0:11]

        os.chdir(WorkDir)
        
        progress['value'] = 10
        
        list = glob.glob(os.path.join(GNCDir+'/W_XX-EUMETSAT-Darmstadt,IMG+SAT,MTI1+LI-2-LFL--FD--CHK-BODY---NC4E_C_EUMT_'+(timehrUTC)+'*.nc'))
        list = [elem[len(WorkDir)+1:] for elem in list]
        item = (list[20])
        #print(item[-14:-8])
        item_ID = (item[-14:-8])
        
        for filename in glob.glob(os.path.join(GNCDir+'/W_XX-EUMETSAT-Darmstadt,IMG+SAT,MTI1+LI-2-LFL--FD--CHK-BODY---NC4E_C_EUMT_*'+(item_ID)+'*.nc')):
        #print(filename)  
            shutil.copy(filename, WorkDir)

        progress['value'] = 30
        
        LI_list = []
        LI_list = glob.glob(os.path.join(WorkDir+'/W_XX-EUMETSAT-Darmstadt,IMG+SAT,MTI1+LI-2-LFL--FD--CHK-BODY---NC4E_C_EUMT_*'+(item_ID)+'*.nc'))
        
        li_ds_list = [netCDF4.Dataset(filename, 'r') for filename in LI_list]
        
        for var in li_ds_list[0].variables:
            print(var)
        
        #choose variables in netcdf file
        lat = np.array([])
        lon = np.array([])
        flash_id = np.array([])
        flash_time = np.array([])
        flash_dur = np.array([])
        flash_fp = np.array([])
        rad = np.array([])
        nb_events = np.array([])
        nb_groups = np.array([])
        ff_conf = np.array([])
        
        for li_ds in li_ds_list:
            lat = np.append(lat, li_ds['latitude'][:].flatten())
            lon = np.append(lon, li_ds['longitude'][:].flatten())
            flash_id = np.append(flash_id, li_ds['flash_id'][:].flatten())
            flash_time = np.append(flash_time, li_ds['flash_time'][:].flatten())
            flash_dur = np.append(flash_dur, li_ds['flash_duration'][:].flatten())
            flash_fp = np.append(flash_fp, li_ds['flash_footprint'][:].flatten())
            rad = np.append(rad, li_ds['radiance'][:].flatten())
            nb_events = np.append(nb_events, li_ds['number_of_events'][:].flatten())
            nb_groups = np.append(nb_groups, li_ds['number_of_groups'][:].flatten())
            ff_conf = np.append(ff_conf, li_ds['flash_filter_confidence'][:].flatten())
   
        progress['value'] = 60
        
        #Flash_time is measured as epoch time in products from MTG, recalculated to regular date time        
        def epoch_to_datetime(flash_time):
            flash_time_list = []
            
            for i in flash_time:
                mtg_reference_datetime=datetime.datetime(2000, 1, 1, 0, 0)
                mtg_datetime=mtg_reference_datetime + datetime.timedelta(seconds=i)
                flash_time_list.append(mtg_datetime)

            return flash_time_list
            
        progress['value'] = 80
        
        # Building Panda dataframe
        dataset = pd.DataFrame({
                'lat': lat.flatten(),
                'lon': lon.flatten(),
                'flash_id': flash_id.flatten(),
                'flash_time': epoch_to_datetime(flash_time.flatten()),
                'flash_dur': flash_dur.flatten(),
                'flash_fp': flash_fp.flatten(),
                'rad_scale': rad.flatten(),
                'nb_events': nb_events.flatten(),
                'nb_groups': nb_groups.flatten(),
                'ff_conf': ff_conf.flatten(),
            })
 
        #create shape file
        #ensure all values are of dtype float (e.g. in case there is a missing value!)
        dataset['lon']=dataset['lon'].astype(float)
        dataset['lat']=dataset['lat'].astype(float)
        dataset['flash_id']=dataset['flash_id'].astype(int)
        
        dataset['flash_time']=dataset['flash_time'].astype(str)
        dataset['flash_dur']=dataset['flash_dur'].astype(float)
        dataset['flash_fp']=dataset['flash_fp'].astype(float)       

        dataset['rad_scale']=dataset['rad_scale'].astype(float)
        dataset['nb_events']=dataset['nb_events'].astype(float)
        dataset['nb_groups']=dataset['nb_groups'].astype(float)
        dataset['ff_conf']=dataset['ff_conf'].astype(float)           
 
        
        # Create geometry column
        geometry = [Point(xy) for xy in zip(dataset['lon'], dataset['lat'])]

        # Create GeoDataFrame
        gdf = gpd.GeoDataFrame(dataset, geometry=geometry)
        
        progress['value'] = 80

        # Set the coordinate reference system (CRS), WGS84
        gdf.set_crs(epsg=4326, inplace=True)

        # Write to shapefile
        gdf.to_file(WorkDir+'/'+OutFileName+'.shp')

        #dataset.to_csv(WorkDir+'/'+ OutFileName +'.csv')

        progress['value'] = 100
        button0.config(state='normal')
        button1.config(state='normal')

#redundant files deleted in main menu script

def executeInThread(GNCDir, InFileName, WorkDir, OutFileName):
    button0.config(state='disabled')
    button1.config(state='disabled')
    thread = Thread(target = execute, args = (GNCDir, InFileName, WorkDir, OutFileName))
    thread.start()



# for command=doNothing
def doNothing():
   text="Do nothing button"
   return


# for command=clickthumbn
def clickthumbn():
   novi = Toplevel(app)
   novi.title ('Example of imported MTG LI LFL image (in ILWIS)')
   novi.iconbitmap(gnc_path +'\pics\gnc.ico')
   canvas = Canvas(novi, width = 600, height = 373)
   canvas.pack(expand = YES, fill = BOTH)
   gif1 = PhotoImage(file = gnc_path +'\pics\MTG_LI_LFL.gif')
   canvas.create_image(0, 0,image = gif1, anchor = NW)
   canvas.gif1 = gif1

# for command=clickAbout
def clickAbout(): 
   filewin = Toplevel(app)
   filewin.title ('About MTG Li LFL')
   filewin.iconbitmap(gnc_path +'\pics\gnc.ico')
   filewin.geometry("575x340")
   msg = Message(filewin, text="""MTG LI LFL background:

LI Level 2 Lightning Flashes (LFL) contains LI flashes - MTG - 0 degree: The definition of a flash is shared by LI and GLM; collections of groups that are correlated in space and time within the two windows of 330 milliseconds (temporal window) and 16.5km (space window), respectively. Even if the definition of a flash is not uniform among all lightning location systems, the simple fact that a flash is a collection of groups/strokes correlated in space and time somewhat mitigates the differences in the way different types of lightning sensors interpret different lightning processes. This makes flash datasets of different lightning location systems more comparable than group/stroke datasets. See also: https://data.eumetsat.int/product/EO:EUM:DAT:0691

File name convertion: W_XX-EUMETSAT-Darmstadt,IMG+SAT,MTI1+LI-2-LFL--FD--CHK-BODY---NC4E_C_EUMT_*.nc
Number of segments: 60 body segment are retrieved
Temporal reolution: Every 10 minutes, starting from 00:00 UTC
Initial file format: NetCDF

Data is provided throught EUMETCast reception and EUMDAC (online). For Legal Constraints check the 'Product Navigator' at EUMETSAT available at http://navigator.eumetsat.int
""")
   msg.pack()


#main()
#automatically setting of path when running this script from main menu - uncomment to make active   
gnc_path = os.environ["gnc_popup"]

#check path
print (gnc_path)
app = Tk()
app.title("Meteosat Third Generation Lightning Imager Level 2 Lightning Flashes (LFL) import routine")
app.iconbitmap(gnc_path +'\pics\gnc.ico')

w = 650 # width for the Tk app
h = 210 # height for the Tk app

app.geometry('%dx%d' % (w, h))
app.resizable(0,0)

labelText1 = StringVar()
labelText1.set ("""Meteosat Third Generation Lightning Imager Level 2 Lightning Flashes (LFL) 
Typical filename: W_XX-EUMETSAT-Darmstadt,IMG+SAT,MTI1+LI-2-LFL--FD--CHK-BODY---NC4E_C_EUMT_*.nc""")

label1 = Label(app, textvariable=labelText1, anchor='w', justify = LEFT)
label1.pack(side = LEFT)
label1.place(x = 10, y = 5, width=650, height=35)

L1 = Label(app, text="Input Directory:", anchor='w')
L1.pack(side = LEFT)
L1.place(x = 10, y = 50, width=150, height=25)

inputDir = StringVar()
E1 = Entry(app, textvariable=inputDir, bd =2)
E1.pack()
E1.place(x = 165, y = 50, width=445, height=25)

button_dirin = Button(app, text="....", width=2, command = lambda: inputDir.set(askdirectory(initialdir = inputDir.get())))
button_dirin.pack()
button_dirin.place(x = 615, y = 49)

L2 = Label(app, text="Time (yyyymmddhhmm):", anchor='w')
L2.pack(side = LEFT)
L2.place(x = 10, y = 80, width=150, height=25)

inputFile = StringVar()
E2 = Entry(app, textvariable=inputFile, bd =2)
E2.pack()
E2.place(x = 165, y = 80, width=445, height=25)

#button_filein = Button(app, text="....", width=2, command = lambda: inputFile.set(askopenfilename(initialdir = inputDir.get()).split('/')[-1]))
#button_filein.pack()
#button_filein.place(x = 615, y = 79)

L3 = Label(app, text="Output Directory:", anchor='w')
L3.pack(side = LEFT)
L3.place(x = 10, y = 110, width=150, height=25)

outputDir = StringVar()
E3 = Entry(app, textvariable=outputDir, bd =2)
E3.pack()
E3.place(x = 165, y = 110, width=445, height=25)

button_dirout = Button(app, text="....", width=2, command = lambda: outputDir.set(askdirectory(initialdir = outputDir.get())))
button_dirout.pack()
button_dirout.place(x = 615, y = 109) 

L4 = Label(app, text="Output Filename:", anchor='w')
L4.pack(side = LEFT)
L4.place(x = 10, y = 140, width=150, height=25)

outputFile = StringVar()
E4 = Entry(app, textvariable=outputFile, bd =2)
E4.pack()
E4.place(x = 165, y = 140, width=445, height=25)

button0 = Button(app, text="Quit", width=15, command=app.destroy)
button0.pack()
button0.place(x = 525, y = 180)

button1 = Button(app, text="Execute", width=15, command=lambda: executeInThread(inputDir.get(), inputFile.get(), outputDir.get(), outputFile.get()))
button1.pack()
button1.place(x = 405, y = 180)

button2 = Button(app, text="About MTG-LI-LFL", width=15, command=clickAbout)
button2.pack()
button2.place(x = 285, y = 180)

button3 = Button(app, text="Thumbnail", width=15, command=clickthumbn)
button3.pack()
button3.place(x = 165, y = 180)


style = Style()
style.theme_use('clam')
style.configure('blue.Horizontal.TProgressbar', foreground='blue', background='blue')

progress = Progressbar(app, style='blue.Horizontal.TProgressbar', orient=HORIZONTAL, length=100, mode='determinate')
progress.pack()
progress.place(x = 10, y = 180)


app.mainloop()

print("inputFile=" + inputFile.get())
print("outputDir=" + outputDir.get())
print("outputFile=" + outputFile.get())



