#Global GPM Precipitation Product import routine (for a given hour)
#Import GPM Level 2 rainfall (mm/hr)

       
from tkinter import *
from tkinter.filedialog import askdirectory
from tkinter.filedialog import askopenfilename
from tkinter.ttk import Progressbar, Style
import os
import ilwis
import numpy as np
import subprocess
import shutil
import glob
import bz2
from pybufrkit.decoder import Decoder
from pybufrkit.mdquery import MetadataExprParser, MetadataQuerent
from pybufrkit.dataquery import NodePathParser, DataQuerent
import pandas as pd
import geopandas as gpd
from shapely.geometry import Point
from threading import Thread


#passing parameters from sub menu

def execute(GNCDir, InFileName, WorkDir, OutFileName):
    if len(GNCDir) > 0 and len(InFileName) > 0 and len(WorkDir) > 0 and len(OutFileName) > 0:

        progress['value'] = 0

                
        os.chdir(WorkDir)
        ilwis.setWorkingCatalog(WorkDir)
        
        progress['value'] = 10  
        
        #filename = W_XX-EUMETSAT-Darmstadt,SOUNDING+SATELLITE,GPM+GMI+S2_C_EUMP_20250512222300_222327_222825_L1C.bin.bz2
        for filename in glob.glob(os.path.join(GNCDir+'/W_XX-EUMETSAT-Darmstadt,SOUNDING+SATELLITE,GPM+GMI+S2_C_EUMP_'+str(InFileName)+'00_*_L1C.bin.bz2')):
            #print(filename)
            shutil.copy(filename, WorkDir)

        progress['value'] = 20  
 
        for fnames in glob.glob(os.path.join(WorkDir+'/W_XX-EUMETSAT-Darmstadt,SOUNDING+SATELLITE,GPM+GMI+S2_C_EUMP_'+str(InFileName)+'00_*_L1C.bin.bz2')):
            file_in = fnames

        fname = file_in.rsplit('.',1)[0] #get file name without bz2 extension
        with bz2.open(file_in,"rb") as f_in, open(fname,"wb") as f_out:
            f_out.write(f_in.read())  
        
        for fnames in glob.glob(os.path.join(WorkDir+'/W_XX-EUMETSAT-Darmstadt,SOUNDING+SATELLITE,GPM+GMI+S2_C_EUMP_'+str(InFileName)+'00_*_L1C.bin')):
            file_in = fnames
        
        #remove bz file
        list = glob.glob(os.path.join(WorkDir+'/W_XX-EUMETSAT-Darmstadt,SOUNDING+SATELLITE,GPM+GMI+S2_C_EUMP_'+str(InFileName)+'00_*_L1C.bin.bz2')) 
        list = [elem[len(WorkDir)+1:] for elem in list]
        for elem in list:
            os.remove(WorkDir+'/'+elem) 
        
        #redirect popup MSDOS window when executing the sub process
        si = subprocess.STARTUPINFO()
        si.dwFlags |= subprocess.STARTF_USESHOWWINDOW
        
        #split all messages contained in BUFR file into seperate messages
        decode = subprocess.run(['pybufrkit', 'split', file_in], startupinfo=si)
        
        #read all messages and append 
        messages = []
        decoder = Decoder()
        path = file_in
        filelist = glob.glob(path + '.*')
        for file in filelist:
            with open((file), 'rb') as ins:
                bufr_message = decoder.process(ins.read())
                messages.append(bufr_message)
        
        # Query the metadata
        n_subsets = MetadataQuerent(MetadataExprParser()).query(bufr_message, '%n_subsets')
        print(n_subsets)
        
        #Relevant coordinate variables are: 4-latitude (code 005001), 5-longitude (code 006001)
        #Relevant parameters are: Brightness Temperatures (code 012063): S2 has channels 10-13: 165V 165H 183+/-3V 183+/-8V. respectively.  


        progress['value'] = 50
        
        #S2 has BT channels 10-13: 165V 165H 183+/-3V 183+/-8V. 
        
        lats = np.array([])
        lons = np.array([])
        bt10 = np.array([])
        bt11 = np.array([])
        bt12 = np.array([])
        bt13 = np.array([])


        for  message in messages:
            LAT_result = DataQuerent(NodePathParser()).query(message, '005001')
            lats=np.concatenate((lats, np.array([LAT_result.results[key][1] for key in LAT_result.results.keys()])))
            LON_result = DataQuerent(NodePathParser()).query(message, '006001')
            lons=np.concatenate((lons,np.array([LON_result.results[key][1] for key in LON_result.results.keys()])))
            
            BT10 = DataQuerent(NodePathParser()).query(message, '012063')
            bt10 = np.concatenate((bt10,np.array([BT10.results[key][0][0][0] for key in BT10.results.keys()])))
            BT11 = DataQuerent(NodePathParser()).query(message, '012063')
            bt11 = np.concatenate((bt11,np.array([BT11.results[key][0][1][0] for key in BT11.results.keys()])))

            BT12 = DataQuerent(NodePathParser()).query(message, '012063')
            bt12 = np.concatenate((bt12,np.array([BT12.results[key][0][2][0] for key in BT12.results.keys()])))
            BT13 = DataQuerent(NodePathParser()).query(message, '012063')
            bt13 = np.concatenate((bt13,np.array([BT13.results[key][0][3][0] for key in BT13.results.keys()])))        
        
        progress['value'] = 70
        
        # convert numpy array to pandas dataframe -  S2 has BT channels 10-13: 165V 165H 183+/-3V 183+/-8V.
        dataset = pd.DataFrame({'LAT': lats, 'LON': lons, 'TB10_165V': bt10,  'TB11_165H': bt11, 'TB12_183_dev3V': bt12,  'TB13_183_dev8V': bt13})         
        
        #ensure all values are of dtype float (e.g. in case there is a missing value!)
        dataset['TB10_165V']=dataset['TB10_165V'].astype(float)
        dataset['TB11_165H']=dataset['TB11_165H'].astype(float)
        dataset['TB12_183_dev3V']=dataset['TB12_183_dev3V'].astype(float)
        dataset['TB13_183_dev8V']=dataset['TB13_183_dev8V'].astype(float)
        #print(dataset.dtypes)
        
        progress['value'] = 80
        
        # Create geometry column
        geometry = [Point(xy) for xy in zip(dataset['LON'], dataset['LAT'])]

        # Create GeoDataFrame
        gdf = gpd.GeoDataFrame(dataset, geometry=geometry)

        # Set the coordinate reference system (CRS), WGS84
        gdf.set_crs(epsg=4326, inplace=True)

        # Write to shapefile
        gdf.to_file(WorkDir+'/'+OutFileName+'.shp')
        
        progress['value'] = 100
        
        button0.config(state='normal')
        button1.config(state='normal')

#redundant files deleted in main menu script

def executeInThread(GNCDir, InFileName, WorkDir, OutFileName):
    button0.config(state='disabled')
    button1.config(state='disabled')
    thread = Thread(target = execute, args = (GNCDir, InFileName, WorkDir, OutFileName))
    thread.start()



# for command=doNothing
def doNothing():
   text="Do nothing button"
   return


# for command=clickthumbn
def clickthumbn():
   novi = Toplevel(app)
   novi.title ('Example of imported 5 minutes GPM-GMI S2 BT-channel 10 (165Ghz-Ver) image (in ILWIS)')
   novi.iconbitmap(gnc_path +'\pics\gnc.ico')
   canvas = Canvas(novi, width = 505, height = 400)
   canvas.pack(expand = YES, fill = BOTH)
   gif1 = PhotoImage(file = gnc_path +'\pics\P3_GPM_GMI_S2_B10.gif')
   canvas.create_image(0, 0,image = gif1, anchor = NW)
   canvas.gif1 = gif1

# for command=clickAbout
def clickAbout(): 
   filewin = Toplevel(app)
   filewin.title ('About GPM-GMI S2')
   filewin.iconbitmap(gnc_path +'\pics\gnc.ico')
   filewin.geometry("575x340")
   msg = Message(filewin, text="""GPM-GMI S2 background:

GMI Level 1C Brightness Temperature - GPM Core. L1C data from the GMI instrument on the GPM-Core satellite. The Global Precipitation Measurement (GPM) Microwave Imager (GMI) instrument is a multi-channel, conical- scanning, microwave radiometer serving an essential role in the near-global-coverage and frequent-revisit-time requirements of GPM. The GMI is characterized by thirteen microwave channels ranging in frequency from 10 GHz to 183 GHz. S2 has BT channels 10-13: 165V 165H 183+/-3V 183+/-8V respectively. Output created is a shape file containing the 4 channel BT's.

File name convertion: W_XX-EUMETSAT-Darmstadt,SOUNDING+SATELLITE,GPM+GMI+S2_C_EUMP_20250512222300_222327_222825_L1C.bin.bz2
Number of segments: 1 file
Temporal Resolution: 5 minutes resolution
Initial file format: netcdf4

Data is provided throught EUMETCast reception. For Legal Constraints check the 'Product Navigator' at EUMETSAT available at http://navigator.eumetsat.int
""")
   msg.pack()


#main()
#automatically setting of path when running this script from main menu - uncomment to make active   
gnc_path = os.environ["gnc_popup"]

#check path
print (gnc_path)
app = Tk()
app.title("GPM GMI S2 import routine")
app.iconbitmap(gnc_path +'\pics\gnc.ico')

w = 650 # width for the Tk app
h = 210 # height for the Tk app

app.geometry('%dx%d' % (w, h))
app.resizable(0,0)

labelText1 = StringVar()
labelText1.set ("""GPM GMI S2
Typ.fname: W_XX-EUMETSAT-D*stadt,SOUNDING+SATELLITE,GPM+GMI+S2_C_EUMP_202505122223*_*_*_L1C.bin.bz2""")

label1 = Label(app, textvariable=labelText1, anchor='w', justify = LEFT)
label1.pack(side = LEFT)
label1.place(x = 10, y = 5, width=650, height=35)

L1 = Label(app, text="Input Directory:", anchor='w')
L1.pack(side = LEFT)
L1.place(x = 10, y = 50, width=150, height=25)

inputDir = StringVar()
E1 = Entry(app, textvariable=inputDir, bd =2)
E1.pack()
E1.place(x = 165, y = 50, width=445, height=25)

button_dirin = Button(app, text="....", width=2, command = lambda: inputDir.set(askdirectory(initialdir = inputDir.get())))
button_dirin.pack()
button_dirin.place(x = 615, y = 49)

L2 = Label(app, text="Time Stamp (in UTC-time):", anchor='w')
L2.pack(side = LEFT)
L2.place(x = 10, y = 80, width=150, height=25)

inputFile = StringVar()
E2 = Entry(app, textvariable=inputFile, bd =2)
E2.pack()
E2.place(x = 165, y = 80, width=445, height=25)

#button_filein = Button(app, text="....", width=2, command = lambda: inputFile.set(askopenfilename(initialdir = inputDir.get()).split('/')[-1]))
#button_filein.pack()
#button_filein.place(x = 615, y = 79)

L3 = Label(app, text="Output Directory:", anchor='w')
L3.pack(side = LEFT)
L3.place(x = 10, y = 110, width=150, height=25)

outputDir = StringVar()
E3 = Entry(app, textvariable=outputDir, bd =2)
E3.pack()
E3.place(x = 165, y = 110, width=445, height=25)

button_dirout = Button(app, text="....", width=2, command = lambda: outputDir.set(askdirectory(initialdir = outputDir.get())))
button_dirout.pack()
button_dirout.place(x = 615, y = 109) 

L4 = Label(app, text="Output Filename:", anchor='w')
L4.pack(side = LEFT)
L4.place(x = 10, y = 140, width=150, height=25)

outputFile = StringVar()
E4 = Entry(app, textvariable=outputFile, bd =2)
E4.pack()
E4.place(x = 165, y = 140, width=445, height=25)

button0 = Button(app, text="Quit", width=15, command=app.destroy)
button0.pack()
button0.place(x = 525, y = 180)

button1 = Button(app, text="Execute", width=15, command=lambda: executeInThread(inputDir.get(), inputFile.get(), outputDir.get(), outputFile.get()))
button1.pack()
button1.place(x = 405, y = 180)

button2 = Button(app, text="About GMI", width=15, command=clickAbout)
button2.pack()
button2.place(x = 285, y = 180)

button3 = Button(app, text="Thumbnail", width=15, command=clickthumbn)
button3.pack()
button3.place(x = 165, y = 180)


style = Style()
style.theme_use('clam')
style.configure('blue.Horizontal.TProgressbar', foreground='blue', background='blue')

progress = Progressbar(app, style='blue.Horizontal.TProgressbar', orient=HORIZONTAL, length=100, mode='determinate')
progress.pack()
progress.place(x = 10, y = 180)


app.mainloop()

print("inputFile=" + inputFile.get())
print("outputDir=" + outputDir.get())
print("outputFile=" + outputFile.get())



