#Global GPM Precipitation Product import routine (for a given hour)
#Import GPM Level 2 rainfall (mm/hr)

       
from tkinter import *
from tkinter.filedialog import askdirectory
from tkinter.filedialog import askopenfilename
from tkinter.ttk import Progressbar, Style
import os
import ilwis
import numpy as np
import subprocess
import shutil
import glob
from netCDF4 import Dataset
import pandas as pd
import geopandas as gpd
from shapely.geometry import Point
from threading import Thread


#passing parameters from sub menu

def execute(GNCDir, InFileName, WorkDir, OutFileName):
    if len(GNCDir) > 0 and len(InFileName) > 0 and len(WorkDir) > 0 and len(OutFileName) > 0:

        progress['value'] = 0
        timeA = InFileName[:8]
        timeB = InFileName[8:10]

                
        os.chdir(WorkDir)
        ilwis.setWorkingCatalog(WorkDir)
        
        progress['value'] = 10  
        
        #filename = #2A.GPM.GMI.GPROF2021v1.20250509-S225327-E225825.V07C.RT-H5
        for filename in glob.glob(os.path.join(GNCDir+'/2A.GPM.GMI.GPROF2021*.'+str(timeA)+'-S'+str(timeB)+'*-E*.RT-H5')):
            #print(filename)
            shutil.copy(filename, WorkDir)

        progress['value'] = 20  

        fnames = glob.glob(WorkDir + '/2A.GPM.GMI.GPROF2021*.'+str(timeA)+'-S'+str(timeB)+'*-E*.RT-H5')
                    
        ### loop through GPM files (12 per hour) and retrieve lat, lon and Surface Precipitation
        GPM_lon = np.array([])

        for GPM in (fnames):
            GPM = Dataset(GPM, 'r')
            lon = GPM.groups['S1'].variables['Longitude'][:].flatten()
            GPM_lon = np.append(GPM_lon, lon)

                
        progress['value'] = 40  
        
        GPM_lat = np.array([])

        for GPM in (fnames):
            GPM = Dataset(GPM, 'r')
            lat = GPM.groups['S1'].variables['Latitude'][:].flatten()
            GPM_lat = np.append(GPM_lat, lat)
        
        GPM_pcp = np.array([])

        for GPM in (fnames):
            GPM = Dataset(GPM, 'r')
            pcp = GPM.groups['S1'].variables['surfacePrecipitation'][:].flatten()
            GPM_pcp = np.append(GPM_pcp, pcp)
        
        progress['value'] = 60 

        dataset_hour = pd.DataFrame({'LON':GPM_lon, 'LAT':GPM_lat,'PCP':GPM_pcp})
       
        progress['value'] = 80
        
        # Create geometry column
        geometry = [Point(xy) for xy in zip(dataset_hour['LON'], dataset_hour['LAT'])]

        # Create GeoDataFrame
        gdf = gpd.GeoDataFrame(dataset_hour, geometry=geometry)

        # Set the coordinate reference system (CRS), WGS84
        gdf.set_crs(epsg=4326, inplace=True)

        # Write to shapefile
        gdf.to_file(WorkDir+'/'+OutFileName+'.shp')
        
        progress['value'] = 100
        
        button0.config(state='normal')
        button1.config(state='normal')

#redundant files deleted in main menu script

def executeInThread(GNCDir, InFileName, WorkDir, OutFileName):
    button0.config(state='disabled')
    button1.config(state='disabled')
    thread = Thread(target = execute, args = (GNCDir, InFileName, WorkDir, OutFileName))
    thread.start()



# for command=doNothing
def doNothing():
   text="Do nothing button"
   return


# for command=clickthumbn
def clickthumbn():
   novi = Toplevel(app)
   novi.title ('Example of imported GPM hourly Surface Precipitation image (in ILWIS)')
   novi.iconbitmap(gnc_path +'\pics\gnc.ico')
   canvas = Canvas(novi, width = 854, height = 400)
   canvas.pack(expand = YES, fill = BOTH)
   gif1 = PhotoImage(file = gnc_path +'\pics\P3_GPM_PCP.gif')
   canvas.create_image(0, 0,image = gif1, anchor = NW)
   canvas.gif1 = gif1

# for command=clickAbout
def clickAbout(): 
   filewin = Toplevel(app)
   filewin.title ('About GPM PCP')
   filewin.iconbitmap(gnc_path +'\pics\gnc.ico')
   filewin.geometry("575x340")
   msg = Message(filewin, text="""GPM PCP background:

GMI GPROF Precipitation - GPM-Core. L2 NRT rain products generated with the GPROF algorithm from GMI data. Routine extracts data for a given hour, output is a shape file.

File name convertion: 2A.GPM.GMI.GPROF2021v1.20250509-S225327-E225825.V07C.RT-H5
Number of segments: 12files per hour
Temporal Resolution: hourly resolution
Initial file format: netcdf4

Data is provided throught EUMETCast reception. For Legal Constraints check the 'Product Navigator' at EUMETSAT available at http://navigator.eumetsat.int
""")
   msg.pack()


#main()
#automatically setting of path when running this script from main menu - uncomment to make active   
gnc_path = os.environ["gnc_popup"]

#check path
print (gnc_path)
app = Tk()
app.title("GPM hourly PCP import routine")
app.iconbitmap(gnc_path +'\pics\gnc.ico')

w = 650 # width for the Tk app
h = 210 # height for the Tk app

app.geometry('%dx%d' % (w, h))
app.resizable(0,0)

labelText1 = StringVar()
labelText1.set ("""GPM PCP
Typical filename: 2A.GPM.GMI.GPROF2021v1.20250509-S225327-E225825.V07C.RT-H5""")

label1 = Label(app, textvariable=labelText1, anchor='w', justify = LEFT)
label1.pack(side = LEFT)
label1.place(x = 10, y = 5, width=650, height=35)

L1 = Label(app, text="Input Directory:", anchor='w')
L1.pack(side = LEFT)
L1.place(x = 10, y = 50, width=150, height=25)

inputDir = StringVar()
E1 = Entry(app, textvariable=inputDir, bd =2)
E1.pack()
E1.place(x = 165, y = 50, width=445, height=25)

button_dirin = Button(app, text="....", width=2, command = lambda: inputDir.set(askdirectory(initialdir = inputDir.get())))
button_dirin.pack()
button_dirin.place(x = 615, y = 49)

L2 = Label(app, text="Time Stamp (in UTC-time):", anchor='w')
L2.pack(side = LEFT)
L2.place(x = 10, y = 80, width=150, height=25)

inputFile = StringVar()
E2 = Entry(app, textvariable=inputFile, bd =2)
E2.pack()
E2.place(x = 165, y = 80, width=445, height=25)

#button_filein = Button(app, text="....", width=2, command = lambda: inputFile.set(askopenfilename(initialdir = inputDir.get()).split('/')[-1]))
#button_filein.pack()
#button_filein.place(x = 615, y = 79)

L3 = Label(app, text="Output Directory:", anchor='w')
L3.pack(side = LEFT)
L3.place(x = 10, y = 110, width=150, height=25)

outputDir = StringVar()
E3 = Entry(app, textvariable=outputDir, bd =2)
E3.pack()
E3.place(x = 165, y = 110, width=445, height=25)

button_dirout = Button(app, text="....", width=2, command = lambda: outputDir.set(askdirectory(initialdir = outputDir.get())))
button_dirout.pack()
button_dirout.place(x = 615, y = 109) 

L4 = Label(app, text="Output Filename:", anchor='w')
L4.pack(side = LEFT)
L4.place(x = 10, y = 140, width=150, height=25)

outputFile = StringVar()
E4 = Entry(app, textvariable=outputFile, bd =2)
E4.pack()
E4.place(x = 165, y = 140, width=445, height=25)

button0 = Button(app, text="Quit", width=15, command=app.destroy)
button0.pack()
button0.place(x = 525, y = 180)

button1 = Button(app, text="Execute", width=15, command=lambda: executeInThread(inputDir.get(), inputFile.get(), outputDir.get(), outputFile.get()))
button1.pack()
button1.place(x = 405, y = 180)

button2 = Button(app, text="About GPM", width=15, command=clickAbout)
button2.pack()
button2.place(x = 285, y = 180)

button3 = Button(app, text="Thumbnail", width=15, command=clickthumbn)
button3.pack()
button3.place(x = 165, y = 180)


style = Style()
style.theme_use('clam')
style.configure('blue.Horizontal.TProgressbar', foreground='blue', background='blue')

progress = Progressbar(app, style='blue.Horizontal.TProgressbar', orient=HORIZONTAL, length=100, mode='determinate')
progress.pack()
progress.place(x = 10, y = 180)


app.mainloop()

print("inputFile=" + inputFile.get())
print("outputDir=" + outputDir.get())
print("outputFile=" + outputFile.get())



