#SMOS Global Soil Moisture Product import routine (for a given day)
       
from tkinter import *
from tkinter.filedialog import askdirectory
from tkinter.filedialog import askopenfilename
from tkinter.ttk import Progressbar, Style
import os
import ilwis
import numpy as np
import subprocess
import shutil
import glob
from netCDF4 import Dataset
import pandas as pd
import geopandas as gpd
from shapely.geometry import Point
from threading import Thread


#passing parameters from sub menu

def execute(GNCDir, InFileName, WorkDir, OutFileName):
    if len(GNCDir) > 0 and len(InFileName) > 0 and len(WorkDir) > 0 and len(OutFileName) > 0:

        progress['value'] = 0
        time_stamp_in = InFileName
        timeA = time_stamp_in[:8]

                
        os.chdir(WorkDir)
        ilwis.setWorkingCatalog(WorkDir)
        
        #filename = #W_XX-ESA,SMOS,NRTNN_C_LEMM_20250507103424_20250507080804_20250507095804_o_v300_l2sm.nc
        for filename in glob.glob(os.path.join(GNCDir+'/W_XX-ESA,SMOS,NRTNN_C_LEMM_'+timeA+'*_l2sm.nc')):
            #print(filename)
            shutil.copy(filename, WorkDir)

        progress['value'] = 20  

        fnames = glob.glob(WorkDir + '/W_XX-ESA,SMOS,NRTNN_C_LEMM_'+timeA+'*_l2sm.nc')
                    
        ### loop through SMOS files (15) and retrieve lat, lon and soil moisture
        SMOS_lon = np.array([])
        for SMOS in (fnames):
            SMOS = Dataset(SMOS, 'r')
            lon = SMOS.variables['longitude'][:].data
            SMOS_lon = np.append(SMOS_lon, lon)
                
        progress['value'] = 40  
        
        SMOS_lat = np.array([])
        for SMOS in (fnames):
            SMOS = Dataset(SMOS, 'r')
            lat = SMOS.variables['latitude'][:].data
            SMOS_lat = np.append(SMOS_lat, lat)
        
        SMOS_sm = np.array([])
        for SMOS in (fnames):
            SMOS = Dataset(SMOS, 'r')
            sm = SMOS.variables['soil_moisture'][:].data
            SMOS_sm = np.append(SMOS_sm, sm)
        
        progress['value'] = 60 

        dataset_day = pd.DataFrame({'LON':SMOS_lon, 'LAT':SMOS_lat,'SM':SMOS_sm})
       
        progress['value'] = 80
        
        # Create geometry column
        geometry = [Point(xy) for xy in zip(dataset_day['LON'], dataset_day['LAT'])]

        # Create GeoDataFrame
        gdf = gpd.GeoDataFrame(dataset_day, geometry=geometry)

        # Set the coordinate reference system (CRS), WGS84
        gdf.set_crs(epsg=4326, inplace=True)

        # Write to shapefile
        gdf.to_file(WorkDir+'/'+OutFileName+'.shp')
        
        progress['value'] = 100
        
        button0.config(state='normal')
        button1.config(state='normal')

#redundant files deleted in main menu script

def executeInThread(GNCDir, InFileName, WorkDir, OutFileName):
    button0.config(state='disabled')
    button1.config(state='disabled')
    thread = Thread(target = execute, args = (GNCDir, InFileName, WorkDir, OutFileName))
    thread.start()



# for command=doNothing
def doNothing():
   text="Do nothing button"
   return


# for command=clickthumbn
def clickthumbn():
   novi = Toplevel(app)
   novi.title ('Example of imported SMOS Soil Moisture image (in ILWIS)')
   novi.iconbitmap(gnc_path +'\pics\gnc.ico')
   canvas = Canvas(novi, width = 1070, height = 403)
   canvas.pack(expand = YES, fill = BOTH)
   gif1 = PhotoImage(file = gnc_path +'\pics\P3_SMOS_SM.gif')
   canvas.create_image(0, 0,image = gif1, anchor = NW)
   canvas.gif1 = gif1

# for command=clickAbout
def clickAbout(): 
   filewin = Toplevel(app)
   filewin.title ('About SMOS SM')
   filewin.iconbitmap(gnc_path +'\pics\gnc.ico')
   filewin.geometry("575x340")
   msg = Message(filewin, text="""SMOS SM background:

SMOS NRT Soil Moisture product. SMOS NRT-NN Soil Mosture product provides a Near Real Time soil moisture dataset based on the statistical coefficients estimated by a Neural Network. It is provided in the SMOS DGG grid and only at the satellite track. It also provides an estimation of the uncertainty of the estimated soil moisture product and the probability that a soil moisture value is contaminated by Radio Frequency Interference. It is distributed in NetCDF format and it is intended to be used for a wide range of applications needing NRT information on soil moisture. The routine aggregates the files on a daily basis.

File name convertion: W_XX-ESA,SMOS,NRTNN_C_LEMM_20250506092901_20250506065624_20250506085624_o_v300_l2sm.nc
Number of segments: 14 to 30 files per day
Temporal Resolution: daily resolution
Initial file format: netcdf4

Data is provided throught EUMETCast reception. For Legal Constraints check the 'Product Navigator' at EUMETSAT available at http://navigator.eumetsat.int
""")
   msg.pack()


#main()
#automatically setting of path when running this script from main menu - uncomment to make active   
gnc_path = os.environ["gnc_popup"]

#check path
print (gnc_path)
app = Tk()
app.title("SMOS SM-daily import routine")
app.iconbitmap(gnc_path +'\pics\gnc.ico')

w = 650 # width for the Tk app
h = 210 # height for the Tk app

app.geometry('%dx%d' % (w, h))
app.resizable(0,0)

labelText1 = StringVar()
labelText1.set ("""SMOS SM
Typical filename: W_XX-ESA,SMOS,NRTNN_C_LEMM_20250506092901_20250506065624_20250506085624_o_v300_l2sm.nc""")

label1 = Label(app, textvariable=labelText1, anchor='w', justify = LEFT)
label1.pack(side = LEFT)
label1.place(x = 10, y = 5, width=650, height=35)

L1 = Label(app, text="Input Directory:", anchor='w')
L1.pack(side = LEFT)
L1.place(x = 10, y = 50, width=150, height=25)

inputDir = StringVar()
E1 = Entry(app, textvariable=inputDir, bd =2)
E1.pack()
E1.place(x = 165, y = 50, width=445, height=25)

button_dirin = Button(app, text="....", width=2, command = lambda: inputDir.set(askdirectory(initialdir = inputDir.get())))
button_dirin.pack()
button_dirin.place(x = 615, y = 49)

L2 = Label(app, text="Time Stamp (in UTC-time):", anchor='w')
L2.pack(side = LEFT)
L2.place(x = 10, y = 80, width=150, height=25)

inputFile = StringVar()
E2 = Entry(app, textvariable=inputFile, bd =2)
E2.pack()
E2.place(x = 165, y = 80, width=445, height=25)

#button_filein = Button(app, text="....", width=2, command = lambda: inputFile.set(askopenfilename(initialdir = inputDir.get()).split('/')[-1]))
#button_filein.pack()
#button_filein.place(x = 615, y = 79)

L3 = Label(app, text="Output Directory:", anchor='w')
L3.pack(side = LEFT)
L3.place(x = 10, y = 110, width=150, height=25)

outputDir = StringVar()
E3 = Entry(app, textvariable=outputDir, bd =2)
E3.pack()
E3.place(x = 165, y = 110, width=445, height=25)

button_dirout = Button(app, text="....", width=2, command = lambda: outputDir.set(askdirectory(initialdir = outputDir.get())))
button_dirout.pack()
button_dirout.place(x = 615, y = 109) 

L4 = Label(app, text="Output Filename:", anchor='w')
L4.pack(side = LEFT)
L4.place(x = 10, y = 140, width=150, height=25)

outputFile = StringVar()
E4 = Entry(app, textvariable=outputFile, bd =2)
E4.pack()
E4.place(x = 165, y = 140, width=445, height=25)

button0 = Button(app, text="Quit", width=15, command=app.destroy)
button0.pack()
button0.place(x = 525, y = 180)

button1 = Button(app, text="Execute", width=15, command=lambda: executeInThread(inputDir.get(), inputFile.get(), outputDir.get(), outputFile.get()))
button1.pack()
button1.place(x = 405, y = 180)

button2 = Button(app, text="About SM", width=15, command=clickAbout)
button2.pack()
button2.place(x = 285, y = 180)

button3 = Button(app, text="Thumbnail", width=15, command=clickthumbn)
button3.pack()
button3.place(x = 165, y = 180)


style = Style()
style.theme_use('clam')
style.configure('blue.Horizontal.TProgressbar', foreground='blue', background='blue')

progress = Progressbar(app, style='blue.Horizontal.TProgressbar', orient=HORIZONTAL, length=100, mode='determinate')
progress.pack()
progress.place(x = 10, y = 180)


app.mainloop()

print("inputFile=" + inputFile.get())
print("outputDir=" + outputDir.get())
print("outputFile=" + outputFile.get())



