#VIIRS N20 Global SST and Windspeed Product import routine (for a given file - 10 minutes)
       
from tkinter import *
from tkinter.filedialog import askdirectory
from tkinter.filedialog import askopenfilename
from tkinter.ttk import Progressbar, Style
import os
import ilwis
import numpy as np
import subprocess
import shutil
import glob
import netCDF4 as nc
from netCDF4 import Dataset
from threading import Thread


#passing parameters from sub menu

def execute(GNCDir, InFileName, WorkDir, OutFileName):
    if len(GNCDir) > 0 and len(InFileName) > 0 and len(WorkDir) > 0 and len(OutFileName) > 0:

        progress['value'] = 0
        time_stamp = InFileName
                
        os.chdir(WorkDir)
        ilwis.setWorkingCatalog(WorkDir)
        
        #filename = #20250508003000-OSPO-L3U_GHRSST-SSTsubskin-VIIRS_N20-ACSPO_V2.80-v02.0-fv01.0.nc
        for filename in glob.glob(os.path.join(GNCDir+'/'+str(time_stamp)+'*-OSPO-L3U_GHRSST-SSTsubskin-VIIRS_N20-ACSPO_*.nc')):
            #print(filename)
            shutil.copy(filename, WorkDir)

        progress['value'] = 20  

        for fnames in glob.glob(os.path.join(WorkDir+'/'+str(time_stamp)+'*-OSPO-L3U_GHRSST-SSTsubskin-VIIRS_N20-ACSPO_*.nc')):
            file_in = fnames
            #print(file_in)
                    
        # Import the data for sea surface temperature (sst) and windspeed (ws)
        N20 = Dataset(file_in, 'r')
        
        #set ilwis georeference and empty raster
        grf = ilwis.GeoReference('code=georef:type=corners, csy=epsg:4326, envelope=-180 90 180 -90, gridsize=18000 9000, cornerofcorners=yes')
        dfNum = ilwis.DataDefinition(ilwis.NumericDomain('code=value'), ilwis.NumericRange(0, 500, 0))
        rc = ilwis.RasterCoverage()
        rc.setSize(ilwis.Size(18000,9000,1))
        rc.setGeoReference(grf)
        rc.setDataDef(dfNum)
        
        progress['value'] = 40  
        
        sst = N20['sea_surface_temperature']
        array_2d = np.squeeze(sst)
        
        rc.array2raster(array_2d)
        rc_cor = ilwis.do('mapcalc','iff(@1>=0,@1,?)',rc)
        
        progress['value'] = 60 
        
        # Write sst to file
        rc_cor.store(WorkDir+'/'+OutFileName+'_sst.mpr')

        ws = N20['wind_speed']
        array_2d = np.squeeze(ws)
       
        progress['value'] = 80
        
        rc.array2raster(array_2d)
        rc_cor = ilwis.do('mapcalc','iff(@1>=0,@1,?)',rc)
        
        # Write ws to file
        rc_cor.store(WorkDir+'/'+OutFileName+'_ws.mpr')
        
        progress['value'] = 100
        
        button0.config(state='normal')
        button1.config(state='normal')

#redundant files deleted in main menu script

def executeInThread(GNCDir, InFileName, WorkDir, OutFileName):
    button0.config(state='disabled')
    button1.config(state='disabled')
    thread = Thread(target = execute, args = (GNCDir, InFileName, WorkDir, OutFileName))
    thread.start()



# for command=doNothing
def doNothing():
   text="Do nothing button"
   return


# for command=clickthumbn
def clickthumbn():
   novi = Toplevel(app)
   novi.title ('Example of imported NOAA 20 VIIRS Wind Speed image (in ILWIS)')
   novi.iconbitmap(gnc_path +'\pics\gnc.ico')
   canvas = Canvas(novi, width = 807, height = 400)
   canvas.pack(expand = YES, fill = BOTH)
   gif1 = PhotoImage(file = gnc_path +'\pics\P3_N20_WS.gif')
   canvas.create_image(0, 0,image = gif1, anchor = NW)
   canvas.gif1 = gif1

# for command=clickAbout
def clickAbout(): 
   filewin = Toplevel(app)
   filewin.title ('About NOAA 20')
   filewin.iconbitmap(gnc_path +'\pics\gnc.ico')
   filewin.geometry("575x340")
   msg = Message(filewin, text="""N20 VIIRS background:

VIIRS Sea Surface Temperature / Wind Speed - Multimission. This is the Sea Surface Temperature (SST) and Wind Speed (WS) L3U product from the VIIRS instrument onboard NOAA's SNPP and NOAA-20 satellites. The Advanced Clear-Sky Processor for Oceans (ACSPO) System produces SST in each cloud-free pixel over water. The data are organised in 10 min granule files in NetCDF4 format, compliant with the Group for High-Resolution SST (GHRSST) Data Specifications version 2 (GDS2). Real time U.S. SNPP/NOAA-20 data service to MyOcean2 and MACC-II Copernicus projects, capitalising on the NOAA-EUMETSAT IJPS cooperation. Note suffix created, _sst and _ws respectively.

File name convertion: 20250508212000-OSPO-L3U_GHRSST-SSTsubskin-VIIRS_N20-ACSPO_V2.80-v02.0-fv01.0.nc
Number of segments: 1 segment
Temporal Resolution: 10 minutes
Initial file format: netcdf4

Data is provided throught EUMETCast reception. For Legal Constraints check the 'Product Navigator' at EUMETSAT available at http://navigator.eumetsat.int
""")
   msg.pack()


#main()
#automatically setting of path when running this script from main menu - uncomment to make active   
gnc_path = os.environ["gnc_popup"]

#check path
print (gnc_path)
app = Tk()
app.title("S-NPP VIIRS import routine")
app.iconbitmap(gnc_path +'\pics\gnc.ico')

w = 650 # width for the Tk app
h = 210 # height for the Tk app

app.geometry('%dx%d' % (w, h))
app.resizable(0,0)

labelText1 = StringVar()
labelText1.set ("""S-NPP
Typical filename: 20250508212000-OSPO-L3U_GHRSST-SSTsubskin-VIIRS_N20-ACSPO_V2.80-v02.0-fv01.0.nc""")

label1 = Label(app, textvariable=labelText1, anchor='w', justify = LEFT)
label1.pack(side = LEFT)
label1.place(x = 10, y = 5, width=650, height=35)

L1 = Label(app, text="Input Directory:", anchor='w')
L1.pack(side = LEFT)
L1.place(x = 10, y = 50, width=150, height=25)

inputDir = StringVar()
E1 = Entry(app, textvariable=inputDir, bd =2)
E1.pack()
E1.place(x = 165, y = 50, width=445, height=25)

button_dirin = Button(app, text="....", width=2, command = lambda: inputDir.set(askdirectory(initialdir = inputDir.get())))
button_dirin.pack()
button_dirin.place(x = 615, y = 49)

L2 = Label(app, text="Time Stamp (in UTC-time):", anchor='w')
L2.pack(side = LEFT)
L2.place(x = 10, y = 80, width=150, height=25)

inputFile = StringVar()
E2 = Entry(app, textvariable=inputFile, bd =2)
E2.pack()
E2.place(x = 165, y = 80, width=445, height=25)

#button_filein = Button(app, text="....", width=2, command = lambda: inputFile.set(askopenfilename(initialdir = inputDir.get()).split('/')[-1]))
#button_filein.pack()
#button_filein.place(x = 615, y = 79)

L3 = Label(app, text="Output Directory:", anchor='w')
L3.pack(side = LEFT)
L3.place(x = 10, y = 110, width=150, height=25)

outputDir = StringVar()
E3 = Entry(app, textvariable=outputDir, bd =2)
E3.pack()
E3.place(x = 165, y = 110, width=445, height=25)

button_dirout = Button(app, text="....", width=2, command = lambda: outputDir.set(askdirectory(initialdir = outputDir.get())))
button_dirout.pack()
button_dirout.place(x = 615, y = 109) 

L4 = Label(app, text="Output Filename:", anchor='w')
L4.pack(side = LEFT)
L4.place(x = 10, y = 140, width=150, height=25)

outputFile = StringVar()
E4 = Entry(app, textvariable=outputFile, bd =2)
E4.pack()
E4.place(x = 165, y = 140, width=445, height=25)

button0 = Button(app, text="Quit", width=15, command=app.destroy)
button0.pack()
button0.place(x = 525, y = 180)

button1 = Button(app, text="Execute", width=15, command=lambda: executeInThread(inputDir.get(), inputFile.get(), outputDir.get(), outputFile.get()))
button1.pack()
button1.place(x = 405, y = 180)

button2 = Button(app, text="About N20", width=15, command=clickAbout)
button2.pack()
button2.place(x = 285, y = 180)

button3 = Button(app, text="Thumbnail", width=15, command=clickthumbn)
button3.pack()
button3.place(x = 165, y = 180)


style = Style()
style.theme_use('clam')
style.configure('blue.Horizontal.TProgressbar', foreground='blue', background='blue')

progress = Progressbar(app, style='blue.Horizontal.TProgressbar', orient=HORIZONTAL, length=100, mode='determinate')
progress.pack()
progress.place(x = 10, y = 180)


app.mainloop()

print("inputFile=" + inputFile.get())
print("outputDir=" + outputDir.get())
print("outputFile=" + outputFile.get())



