#MESA JRC -  Monthly water occurrence Product for various regions in Africa import routine (for a given month)
       
from tkinter import *
from tkinter.filedialog import askdirectory
from tkinter.filedialog import askopenfilename
from tkinter.ttk import Progressbar, Style
import os

import numpy as np
import subprocess
import shutil
import glob
import gzip
import tarfile
import zipfile
from datetime import datetime
from osgeo import gdal
import ilwis
from threading import Thread


#passing parameters from sub menu

def execute(GNCDir, InFileName, WorkDir, OutFileName):
    if len(GNCDir) > 0 and len(InFileName) > 0 and len(WorkDir) > 0 and len(OutFileName) > 0:

        progress['value'] = 0
                
        os.chdir(WorkDir)
        ilwis.setWorkingCatalog(WorkDir)
        
        #filename = MESA_JRC_wd-gee_occurr_20250301_WD-GEE-CENTRALAFRICA-AVG_1.0.tgz
        for filename in glob.glob(os.path.join(GNCDir+'/MESA_JRC_wd-gee_occurr_'+(InFileName[:8])+'_WD-GEE-CENTRALAFRICA-AVG_*.tgz')):
            #print(filename)
            shutil.copy(filename, WorkDir)
        
        for fnames in glob.glob(os.path.join(WorkDir+r'/MESA_JRC_wd-gee_occurr_'+(InFileName[:8])+'_WD-GEE-CENTRALAFRICA-AVG_*.tgz')):
            file_in = fnames
            #print(file_in)        
       
        progress['value'] = 20  
        
        # extract the data from the *.tgz
        fname = file_in.rsplit('.',1)[0] #get file name without tgz extension
        tgz_path = fname+'.tgz'
        tar_path = fname+'.tar'

        # Extract the .tar file from .tgz
        with gzip.open(tgz_path, 'rb') as f_in:
            with open(tar_path, 'wb') as f_out:
                shutil.copyfileobj(f_in, f_out)

        #directory to extract to:
        extract_dir = WorkDir

        # Open and extract the .tar archive
        with tarfile.open(tar_path, 'r:') as tar:
            tar.extractall(path=extract_dir)
            print(f"Extracted files to {extract_dir}")            
        
        tifname = file_in.rsplit('.',1)[0] #get file name without tgz extension
        print((tifname)+'.tif')
        
        progress['value'] = 40 
        
        #rename the tif file to its output filename
        os.rename(tifname +'.tif', OutFileName+'.tif')
        
        progress['value'] = 60 
        
        print(WorkDir+'/'+(OutFileName)+'.tif')

        gdal_in = gdal.Open(WorkDir+'/'+(OutFileName)+'.tif')
        gdal_in.GetGeoTransform()
        
        geoTransform = gdal_in.GetGeoTransform()
        minx = geoTransform[0]
        maxy = geoTransform[3]
        maxx = minx + geoTransform[1] * gdal_in.RasterXSize
        miny = maxy + geoTransform[5] * gdal_in.RasterYSize

        crdMin = {'x': minx, 'y': miny}
        crdMax = {'x': maxx, 'y': maxy}
        rcSize = {'Col': gdal_in.RasterXSize, 'Row': gdal_in.RasterYSize}

        a12 = 0
        a21 = 0
        rX = crdMax['x'] - crdMin['x']
        rY = crdMax['y'] - crdMin['y']

        a11 = rcSize['Col'] / rX
        a22 = - rcSize['Row'] / rY
        b1 = - a11 * crdMin['x']
        b2 = - a22 * crdMax['y']
       
        progress['value'] = 80
        
        def writeGrf(filename):
            filename = filename.split('.')[0] + '.grf'
            o = open(filename, 'w')
            o.write('[Ilwis]\n')
            o.write('Description=GeoReference from ' + OutFileName+'\n')
            o.write('Time=' + str(int(datetime.now().timestamp()))+'\n')
            o.write('Version=3.1\n')
            o.write('Class=GeoReference Corners\n')
            o.write('Type=GeoRef\n')
            o.write('[GeoRef]\n')
            o.write('CoordSystem=LatLonWGS84.csy\n')
            o.write('Lines=' + str(rcSize['Row'])+'\n')
            o.write('Columns=' + str(rcSize['Col'])+'\n')
            o.write('Type=GeoRefCorners\n')
            o.write('[GeoRefSmpl]\n')
            o.write('a11=' + str(a11)+'\n')
            o.write('a12=' + str(a12)+'\n')
            o.write('a21=' + str(a21)+'\n')
            o.write('a22=' + str(a22)+'\n')
            o.write('b1=' + str(b1)+'\n')
            o.write('b2=' + str(b2)+'\n')
            o.write('[GeoRefCorners]\n')
            o.write('CornersOfCorners=Yes\n')
            o.write('MinX=' + str(minx)+'\n')
            o.write('MinY=' + str(miny)+'\n')
            o.write('MaxX=' + str(maxx)+'\n')
            o.write('MaxY=' + str(maxy)+'\n')
            o.flush()
            o.close()
            return filename
        
        writeGrf(OutFileName)
        
        def writeMpr(filename):
            filename = filename.split('.')[0] + '.mpr'
            o = open(filename, 'w')
            o.write('[Ilwis]\n')
            o.write('Description=Raster Map from' +OutFileName+', using GDAL, method : use as\n')
            o.write('Time=' + str(int(datetime.now().timestamp()))+'\n')
            o.write('Version=3.1\n')
            o.write('Class=Raster Map\n')
            o.write('UseAs=Yes\n')
            o.write('Type=BaseMap\n')
            o.write('[BaseMap]\n')
            o.write('CoordSystem=LatLonWGS84.csy\n')
            o.write('CoordBounds=' +str(minx)+' '+str(miny)+' '+str(maxx)+' '+str(maxy)+'\n')
            o.write('Domain=Image.dom\n')
            o.write('DomainInfo=Image.dom;Byte;image;0;;\n')
            o.write('Range=0:255:offset=0\n')
            o.write('ThreeDCoordinates=No\n')
            o.write('Type=Map\n')
            o.write('MinMax=100:100\n')
            o.write('[Map]\n')
            o.write('GeoRef='+filename.split('.')[0] +'.grf\n')
            o.write('Size=' + str(rcSize['Row'])+' '+ str(rcSize['Col'])+'\n')
            o.write('Type=MapStore\n')
            o.write('[MapStore]\n')
            o.write('StoreTime=0\n')
            o.write('Structure=Line\n')
            o.write('StartOffset=0\n')
            o.write('RowLength=' + str(rcSize['Row'])+'\n')
            o.write('PixelInterLeaved=Yes\n')
            o.write('SwapBytes=Yes\n')
            o.write('Type=ForeignFormat\n')
            o.write('UseAs=Yes\n')
            o.write('Format=3\n')
            o.write('[ForeignFormat]\n')
            o.write('Input='+filename.split('.')[0] +'.tif\n')
            o.write('AssociatedType=6\n')
            o.write('UseAs=Yes\n')
            o.write('Method=GDAL\n')
            o.write('Layer=1\n')
            o.flush()
            o.close()
            return filename    

        writeMpr(OutFileName)            
        
        progress['value'] = 100
        
        button0.config(state='normal')
        button1.config(state='normal')

#redundant files deleted in main menu script

def executeInThread(GNCDir, InFileName, WorkDir, OutFileName):
    button0.config(state='disabled')
    button1.config(state='disabled')
    thread = Thread(target = execute, args = (GNCDir, InFileName, WorkDir, OutFileName))
    thread.start()



# for command=doNothing
def doNothing():
   text="Do nothing button"
   return


# for command=clickthumbn
def clickthumbn():
   novi = Toplevel(app)
   novi.title ('Example of imported MESA JRC Water occurrence image (in ILWIS)')
   novi.iconbitmap(gnc_path +'\pics\gnc.ico')
   canvas = Canvas(novi, width = 400, height = 571)
   canvas.pack(expand = YES, fill = BOTH)
   gif1 = PhotoImage(file = gnc_path +'\pics\P3_Mesa_JRC_Central.gif')
   canvas.create_image(0, 0,image = gif1, anchor = NW)
   canvas.gif1 = gif1

# for command=clickAbout
def clickAbout(): 
   filewin = Toplevel(app)
   filewin.title ('About MESA JRC')
   filewin.iconbitmap(gnc_path +'\pics\gnc.ico')
   filewin.geometry("575x340")
   msg = Message(filewin, text="""MESA JRC background:

High-resolution Mapping of Global Surface Water - Landsat. The 'High-resolution mapping of global surface water' represents the monthly occurrence of water detection over land, as identified by applying the Global Surface Water algorithm developed by JRC and adopted by UN Environment for monitoring SDG 6.6.1 (see https://global-surface-water.appspot.com). The product is generated by JRC every month, over 5 different African regions, defined in the framework of MESA and GMES&Africa projects with the beneficiary Institutions. The five regions are North Africa, South Africa, Central Africa, ECOWAS (West Africa) and IGAD (East Africa).

File name convertion: MESA_JRC_wd-gee_occurr_20250301_WD-GEE-CENTRALAFRICA-AVG_1.0.tgz
Number of segments: 1 file 
Temporal Resolution: monthly resolution
Initial file format: tif

Data is provided throught EUMETCast reception. For Legal Constraints check the 'Product Navigator' at EUMETSAT available at http://navigator.eumetsat.int
""")
   msg.pack()


#main()
#automatically setting of path when running this script from main menu - uncomment to make active   
gnc_path = os.environ["gnc_popup"]

#check path
print (gnc_path)
app = Tk()
app.title("MESA JRC water occurrence import routine")
app.iconbitmap(gnc_path +'\pics\gnc.ico')

w = 650 # width for the Tk app
h = 210 # height for the Tk app

app.geometry('%dx%d' % (w, h))
app.resizable(0,0)

labelText1 = StringVar()
labelText1.set ("""MESA JRC
Typical filename: MESA_JRC_wd-gee_occurr_20250301_WD-GEE-CENTRALAFRICA-AVG_1.0.tgz""")

label1 = Label(app, textvariable=labelText1, anchor='w', justify = LEFT)
label1.pack(side = LEFT)
label1.place(x = 10, y = 5, width=650, height=35)

L1 = Label(app, text="Input Directory:", anchor='w')
L1.pack(side = LEFT)
L1.place(x = 10, y = 50, width=150, height=25)

inputDir = StringVar()
E1 = Entry(app, textvariable=inputDir, bd =2)
E1.pack()
E1.place(x = 165, y = 50, width=445, height=25)

button_dirin = Button(app, text="....", width=2, command = lambda: inputDir.set(askdirectory(initialdir = inputDir.get())))
button_dirin.pack()
button_dirin.place(x = 615, y = 49)

L2 = Label(app, text="Time Stamp (in UTC-time):", anchor='w')
L2.pack(side = LEFT)
L2.place(x = 10, y = 80, width=150, height=25)

inputFile = StringVar()
E2 = Entry(app, textvariable=inputFile, bd =2)
E2.pack()
E2.place(x = 165, y = 80, width=445, height=25)

#button_filein = Button(app, text="....", width=2, command = lambda: inputFile.set(askopenfilename(initialdir = inputDir.get()).split('/')[-1]))
#button_filein.pack()
#button_filein.place(x = 615, y = 79)

L3 = Label(app, text="Output Directory:", anchor='w')
L3.pack(side = LEFT)
L3.place(x = 10, y = 110, width=150, height=25)

outputDir = StringVar()
E3 = Entry(app, textvariable=outputDir, bd =2)
E3.pack()
E3.place(x = 165, y = 110, width=445, height=25)

button_dirout = Button(app, text="....", width=2, command = lambda: outputDir.set(askdirectory(initialdir = outputDir.get())))
button_dirout.pack()
button_dirout.place(x = 615, y = 109) 

L4 = Label(app, text="Output Filename:", anchor='w')
L4.pack(side = LEFT)
L4.place(x = 10, y = 140, width=150, height=25)

outputFile = StringVar()
E4 = Entry(app, textvariable=outputFile, bd =2)
E4.pack()
E4.place(x = 165, y = 140, width=445, height=25)

button0 = Button(app, text="Quit", width=15, command=app.destroy)
button0.pack()
button0.place(x = 525, y = 180)

button1 = Button(app, text="Execute", width=15, command=lambda: executeInThread(inputDir.get(), inputFile.get(), outputDir.get(), outputFile.get()))
button1.pack()
button1.place(x = 405, y = 180)

button2 = Button(app, text="About JRC", width=15, command=clickAbout)
button2.pack()
button2.place(x = 285, y = 180)

button3 = Button(app, text="Thumbnail", width=15, command=clickthumbn)
button3.pack()
button3.place(x = 165, y = 180)


style = Style()
style.theme_use('clam')
style.configure('blue.Horizontal.TProgressbar', foreground='blue', background='blue')

progress = Progressbar(app, style='blue.Horizontal.TProgressbar', orient=HORIZONTAL, length=100, mode='determinate')
progress.pack()
progress.place(x = 10, y = 180)


app.mainloop()

print("inputFile=" + inputFile.get())
print("outputDir=" + outputDir.get())
print("outputFile=" + outputFile.get())



