#MESA University of Ghana - Significant Wave Height import routine (forecast  for  5 days / daily timestep - 5 layers)
       
from tkinter import *
from tkinter.filedialog import askdirectory
from tkinter.filedialog import askopenfilename
from tkinter.ttk import Progressbar, Style
import os
import ilwis
import numpy as np
import subprocess
import shutil
import glob
import rasterio
from netCDF4 import Dataset
from threading import Thread


#passing parameters from sub menu

def execute(GNCDir, InFileName, WorkDir, OutFileName):
    if len(GNCDir) > 0 and len(InFileName) > 0 and len(WorkDir) > 0 and len(OutFileName) > 0:

        progress['value'] = 10
          
        os.chdir(WorkDir)
        ilwis.setWorkingCatalog(WorkDir)
        
        #filename = MESA-UG_SWH_20250518_20250518_00.tif (5 temporal intervals)
        for filename in glob.glob(os.path.join(GNCDir+'/MESA-UG_SWH_'+str(InFileName[:8])+'_*_00.tif')):
            shutil.copy(filename, WorkDir)

        progress['value'] = 40  

        
        grf = ilwis.GeoReference('code=georef:type=corners, csy=epsg:4326, envelope=-35 40 38 -10, gridsize=877 601, cornerofcorners=yes')
        dfNum = ilwis.DataDefinition(ilwis.NumericDomain('code=value'), ilwis.NumericRange(0, 1000, 0))
        rcNew = ilwis.RasterCoverage()
        rcNew.setSize(ilwis.Size(877,601,5))
        rcNew.setGeoReference(grf)
        rcNew.setDataDef(dfNum)
        
               
        all_layers = []
        filelist = glob.glob(WorkDir+r'/MESA-UG_SWH_'+str(InFileName[:8])+'_*_00.tif')
        filelist.sort()

        for file in filelist:
            print('Reading:',file)
            with rasterio.open(file) as ds:
                data = ds.read(1)
                all_layers.append(data)
        all_layers = np.array(all_layers)
        rcNew.array2raster(all_layers)
            
        progress['value'] = 60 
        
        rc_cor = ilwis.do('mapcalc','iff(@1>=0,@1,?)',rcNew)
        
        progress['value'] = 80
        
        rc_cor.store(WorkDir+'/'+OutFileName+'.mpl')
        
        progress['value'] = 100
        
        button0.config(state='normal')
        button1.config(state='normal')

#redundant files deleted in main menu script

def executeInThread(GNCDir, InFileName, WorkDir, OutFileName):
    button0.config(state='disabled')
    button1.config(state='disabled')
    thread = Thread(target = execute, args = (GNCDir, InFileName, WorkDir, OutFileName))
    thread.start()



# for command=doNothing
def doNothing():
   text="Do nothing button"
   return


# for command=clickthumbn
def clickthumbn():
   novi = Toplevel(app)
   novi.title ('Example of imported Significant Wave Height image (in ILWIS)')
   novi.iconbitmap(gnc_path +'\pics\gnc.ico')
   canvas = Canvas(novi, width = 585, height = 400)
   canvas.pack(expand = YES, fill = BOTH)
   gif1 = PhotoImage(file = gnc_path +'\pics\P3_UG_SWH.gif')
   canvas.create_image(0, 0,image = gif1, anchor = NW)
   canvas.gif1 = gif1

# for command=clickAbout
def clickAbout(): 
   filewin = Toplevel(app)
   filewin.title ('About MESA UG')
   filewin.iconbitmap(gnc_path +'\pics\gnc.ico')
   filewin.geometry("575x340")
   msg = Message(filewin, text="""MESA UG SWH background:

Wave Height Forecast - West Africa. Charts of wave forecast are generated by the University of Ghana Regional Marine Centre from data obtained from the Operational Mercator global Ocean analysis system via the Copernicus programme at a resolution of 1/12 (0.08) degree. Forecasting of ocean weather is done for a 5-day period including a nowcast product for the present day. Daily average of wave forecasts are generated for the 5 days from the 3-hourly products by the University of Ghana Regional Marine Centre. The forecast products disseminated by the Regional Marine Centre are produced daily covering the coastal and oceanic waters of western Africa. Five (5) files are generated daily for the five forecasts days. 

File name convertion: MESA-UG_SWH_20250518_20250518_00.tif
Number of segments: 5 files 
Temporal Resolution: daily resolution
Initial file format: tif

Data is provided throught EUMETCast reception. For Legal Constraints check the 'Product Navigator' at EUMETSAT available at http://navigator.eumetsat.int
""")
   msg.pack()


#main()
#automatically setting of path when running this script from main menu - uncomment to make active   
gnc_path = os.environ["gnc_popup"]

#check path
print (gnc_path)
app = Tk()
app.title("MESA UG import routine")
app.iconbitmap(gnc_path +'\pics\gnc.ico')

w = 650 # width for the Tk app
h = 210 # height for the Tk app

app.geometry('%dx%d' % (w, h))
app.resizable(0,0)

labelText1 = StringVar()
labelText1.set ("""MESA UG SWH
Typical filename: MESA-UG_SWH_20250518_20250518_00.tif""")

label1 = Label(app, textvariable=labelText1, anchor='w', justify = LEFT)
label1.pack(side = LEFT)
label1.place(x = 10, y = 5, width=650, height=35)

L1 = Label(app, text="Input Directory:", anchor='w')
L1.pack(side = LEFT)
L1.place(x = 10, y = 50, width=150, height=25)

inputDir = StringVar()
E1 = Entry(app, textvariable=inputDir, bd =2)
E1.pack()
E1.place(x = 165, y = 50, width=445, height=25)

button_dirin = Button(app, text="....", width=2, command = lambda: inputDir.set(askdirectory(initialdir = inputDir.get())))
button_dirin.pack()
button_dirin.place(x = 615, y = 49)

L2 = Label(app, text="Time Stamp (in UTC-time):", anchor='w')
L2.pack(side = LEFT)
L2.place(x = 10, y = 80, width=150, height=25)

inputFile = StringVar()
E2 = Entry(app, textvariable=inputFile, bd =2)
E2.pack()
E2.place(x = 165, y = 80, width=445, height=25)

#button_filein = Button(app, text="....", width=2, command = lambda: inputFile.set(askopenfilename(initialdir = inputDir.get()).split('/')[-1]))
#button_filein.pack()
#button_filein.place(x = 615, y = 79)

L3 = Label(app, text="Output Directory:", anchor='w')
L3.pack(side = LEFT)
L3.place(x = 10, y = 110, width=150, height=25)

outputDir = StringVar()
E3 = Entry(app, textvariable=outputDir, bd =2)
E3.pack()
E3.place(x = 165, y = 110, width=445, height=25)

button_dirout = Button(app, text="....", width=2, command = lambda: outputDir.set(askdirectory(initialdir = outputDir.get())))
button_dirout.pack()
button_dirout.place(x = 615, y = 109) 

L4 = Label(app, text="Output Filename:", anchor='w')
L4.pack(side = LEFT)
L4.place(x = 10, y = 140, width=150, height=25)

outputFile = StringVar()
E4 = Entry(app, textvariable=outputFile, bd =2)
E4.pack()
E4.place(x = 165, y = 140, width=445, height=25)

button0 = Button(app, text="Quit", width=15, command=app.destroy)
button0.pack()
button0.place(x = 525, y = 180)

button1 = Button(app, text="Execute", width=15, command=lambda: executeInThread(inputDir.get(), inputFile.get(), outputDir.get(), outputFile.get()))
button1.pack()
button1.place(x = 405, y = 180)

button2 = Button(app, text="About SWH", width=15, command=clickAbout)
button2.pack()
button2.place(x = 285, y = 180)

button3 = Button(app, text="Thumbnail", width=15, command=clickthumbn)
button3.pack()
button3.place(x = 165, y = 180)


style = Style()
style.theme_use('clam')
style.configure('blue.Horizontal.TProgressbar', foreground='blue', background='blue')

progress = Progressbar(app, style='blue.Horizontal.TProgressbar', orient=HORIZONTAL, length=100, mode='determinate')
progress.pack()
progress.place(x = 10, y = 180)


app.mainloop()

print("inputFile=" + inputFile.get())
print("outputDir=" + outputDir.get())
print("outputFile=" + outputFile.get())



