#Sentinel-3 OLCI LAI Dekadal import routine. 
       
from tkinter import *
from tkinter.filedialog import askdirectory
from tkinter.filedialog import askopenfilename
from tkinter.ttk import Progressbar, Style
import os
import ilwis
import numpy as np
import subprocess
import shutil
import glob
import zipfile
import netCDF4 as nc
from netCDF4 import Dataset
from threading import Thread


#passing parameters from sub menu

def execute(GNCDir, InFileName, WorkDir, OutFileName):
    if len(GNCDir) > 0 and len(InFileName) > 0 and len(WorkDir) > 0 and len(OutFileName) > 0:

        progress['value'] = 10
          
        os.chdir(WorkDir)
        ilwis.setWorkingCatalog(WorkDir)
        
        #filename = c_gls_LAI300-RT0_202505100000_AFRI_OLCI_V1.1.1.zip 
        for filename in glob.glob(os.path.join(GNCDir+'/c_gls_LAI300-RT0_'+str(InFileName[:8])+'0000_AFRI_OLCI_*.zip')):
            print(filename)
            shutil.copy(filename, WorkDir)
            
        zip_path = os.path.join(WorkDir, 'c_gls_LAI300-RT0_'+str(InFileName[:8])+'0000_AFRI_OLCI_V1.1.1.zip')
        nested_file_path = str(InFileName[:8])+'/c_gls_LAI300-RT0_'+str(InFileName[:8])+'0000_AFRI_OLCI_V1.1.1.nc'

        # Target final output: same folder as the zip
        output_file_path = os.path.join(WorkDir, os.path.basename(nested_file_path))

        # Extract just the nested file
        with zipfile.ZipFile(zip_path, 'r') as zip_ref:
            if nested_file_path in zip_ref.namelist():
                # Extract to temporary location inside WorkDir
                zip_ref.extract(nested_file_path, path=WorkDir)

                # Move extracted file to WorkDir (flatten path)
                extracted_path = os.path.join(WorkDir, nested_file_path)
                shutil.move(extracted_path, output_file_path)

                # Clean up: remove now-empty folder
                nested_folder = os.path.join(WorkDir, str(InFileName[:8]))
                if os.path.isdir(nested_folder) and not os.listdir(nested_folder):
                    os.rmdir(nested_folder)

                print(f"Extracted to: {output_file_path}")
            else:
                print(f"File '{nested_file_path}' not found in ZIP.")            

        progress['value'] = 20  

        #Filename for LAI unzippped = c_gls_LAI300-RT0_202505100000_AFRI_OLCI_V1.1.1.nc
        for filename in glob.glob(os.path.join(WorkDir+r'/c_gls_LAI300-RT0_'+str(InFileName[:8])+'0000_AFRI_OLCI_*.nc')):
            file_in = filename
            #print(file_in)
            
        progress['value'] = 30 
        
        ds = nc.Dataset(file_in)
        data = ds['LAI']
        data_2d = np.squeeze(data)
        olci = data_2d[:]
        
        progress['value'] = 50
        
        #create output map in ILWIS format
        #    NC: 30240  
        #    NL: 26880
        grf = ilwis.GeoReference('code=georef:type=corners, csy=epsg:4326, envelope=-30 40 60 -40, gridsize=30240 26880, cornerofcorners=yes')
        dfNum = ilwis.DataDefinition(ilwis.NumericDomain('code=value'), ilwis.NumericRange(-100, 500, 0))
        rc = ilwis.RasterCoverage()
        rc.setSize(ilwis.Size(30240,26880,1))
        rc.setGeoReference(grf)
        rc.setDataDef(dfNum)

        # change array to ilwis array
        rc.array2raster(olci)
        
        progress['value'] = 80
        
        rc_cor = ilwis.do('mapcalc', 'iff(@1<255, @1, ?)',rc) 
        rc_cor.store(WorkDir+'/'+OutFileName+'.mpr')
        
        progress['value'] = 100
        
        button0.config(state='normal')
        button1.config(state='normal')

#redundant files deleted in main menu script

def executeInThread(GNCDir, InFileName, WorkDir, OutFileName):
    button0.config(state='disabled')
    button1.config(state='disabled')
    thread = Thread(target = execute, args = (GNCDir, InFileName, WorkDir, OutFileName))
    thread.start()



# for command=doNothing
def doNothing():
   text="Do nothing button"
   return


# for command=clickthumbn
def clickthumbn():
   novi = Toplevel(app)
   novi.title ('Example of imported S3-OLCI LAI Africa image (in ILWIS)')
   novi.iconbitmap(gnc_path +'\pics\gnc.ico')
   canvas = Canvas(novi, width = 451, height = 400)
   canvas.pack(expand = YES, fill = BOTH)
   gif1 = PhotoImage(file = gnc_path +'\pics\P3_S3_OLCI_LAI.gif')
   canvas.create_image(0, 0,image = gif1, anchor = NW)
   canvas.gif1 = gif1

# for command=clickAbout
def clickAbout(): 
   filewin = Toplevel(app)
   filewin.title ('About S3-OLCI LAI Africa')
   filewin.iconbitmap(gnc_path +'\pics\gnc.ico')
   filewin.geometry("575x340")
   msg = Message(filewin, text="""S3-OLCI LAI Africa background:

Leaf Area Index - Sentinel-3 - Africa. LAI is defined by CEOS as half the developed area of the convex hull wrapping the green canopy elements per unit horizontal ground. This definition allows accounting for elements which are not flat such as needles or stems. LAI is strongly non-linearly related to reflectance. Therefore, its estimation from remote sensing observations will be scale dependent over heterogeneous landscapes. When observing a canopy made of different layers of vegetation, it is therefore mandatory to consider all the green layers. This is particularly important for forest canopies where the understory may represent a very significant contribution to the total canopy LAI. The derived LAI corresponds therefore to the total green LAI, including the contribution of the green elements of the understory. From the available global product, this product is a geographic subset over Africa. 
  
File name convertion: c_gls_LAI300-RT0_202204200000_AFRI_OLCI_V1.1.1.zip
Number of segments: 1 file 
Temporal Resolution: 3 per month
Initial file format: netCDF

Data is provided throught EUMETCast reception. For Legal Constraints check the 'Product Navigator' at EUMETSAT available at http://navigator.eumetsat.int
""")
   msg.pack()


#main()
#automatically setting of path when running this script from main menu - uncomment to make active   
gnc_path = os.environ["gnc_popup"]

#check path
print (gnc_path)
app = Tk()
app.title("S3-OLCI import routine")
app.iconbitmap(gnc_path +'\pics\gnc.ico')

w = 650 # width for the Tk app
h = 210 # height for the Tk app

app.geometry('%dx%d' % (w, h))
app.resizable(0,0)

labelText1 = StringVar()
labelText1.set ("""S3-OLCI LAI
Typical filename: c_gls_LAI300-RT0_202204200000_AFRI_OLCI_V1.1.1.zip""")

label1 = Label(app, textvariable=labelText1, anchor='w', justify = LEFT)
label1.pack(side = LEFT)
label1.place(x = 10, y = 5, width=650, height=35)

L1 = Label(app, text="Input Directory:", anchor='w')
L1.pack(side = LEFT)
L1.place(x = 10, y = 50, width=150, height=25)

inputDir = StringVar()
E1 = Entry(app, textvariable=inputDir, bd =2)
E1.pack()
E1.place(x = 165, y = 50, width=445, height=25)

button_dirin = Button(app, text="....", width=2, command = lambda: inputDir.set(askdirectory(initialdir = inputDir.get())))
button_dirin.pack()
button_dirin.place(x = 615, y = 49)

L2 = Label(app, text="Time Stamp (in UTC-time):", anchor='w')
L2.pack(side = LEFT)
L2.place(x = 10, y = 80, width=150, height=25)

inputFile = StringVar()
E2 = Entry(app, textvariable=inputFile, bd =2)
E2.pack()
E2.place(x = 165, y = 80, width=445, height=25)

#button_filein = Button(app, text="....", width=2, command = lambda: inputFile.set(askopenfilename(initialdir = inputDir.get()).split('/')[-1]))
#button_filein.pack()
#button_filein.place(x = 615, y = 79)

L3 = Label(app, text="Output Directory:", anchor='w')
L3.pack(side = LEFT)
L3.place(x = 10, y = 110, width=150, height=25)

outputDir = StringVar()
E3 = Entry(app, textvariable=outputDir, bd =2)
E3.pack()
E3.place(x = 165, y = 110, width=445, height=25)

button_dirout = Button(app, text="....", width=2, command = lambda: outputDir.set(askdirectory(initialdir = outputDir.get())))
button_dirout.pack()
button_dirout.place(x = 615, y = 109) 

L4 = Label(app, text="Output Filename:", anchor='w')
L4.pack(side = LEFT)
L4.place(x = 10, y = 140, width=150, height=25)

outputFile = StringVar()
E4 = Entry(app, textvariable=outputFile, bd =2)
E4.pack()
E4.place(x = 165, y = 140, width=445, height=25)

button0 = Button(app, text="Quit", width=15, command=app.destroy)
button0.pack()
button0.place(x = 525, y = 180)

button1 = Button(app, text="Execute", width=15, command=lambda: executeInThread(inputDir.get(), inputFile.get(), outputDir.get(), outputFile.get()))
button1.pack()
button1.place(x = 405, y = 180)

button2 = Button(app, text="About LAI", width=15, command=clickAbout)
button2.pack()
button2.place(x = 285, y = 180)

button3 = Button(app, text="Thumbnail", width=15, command=clickthumbn)
button3.pack()
button3.place(x = 165, y = 180)


style = Style()
style.theme_use('clam')
style.configure('blue.Horizontal.TProgressbar', foreground='blue', background='blue')

progress = Progressbar(app, style='blue.Horizontal.TProgressbar', orient=HORIZONTAL, length=100, mode='determinate')
progress.pack()
progress.place(x = 10, y = 180)


app.mainloop()

print("inputFile=" + inputFile.get())
print("outputDir=" + outputDir.get())
print("outputFile=" + outputFile.get())



