#TAMSAT Monthly Precipitation Anomaly import routine.
       
from tkinter import *
from tkinter.filedialog import askdirectory
from tkinter.filedialog import askopenfilename
from tkinter.ttk import Progressbar, Style
import os
import ilwis
import numpy as np
import subprocess
import shutil
import glob
import rasterio
import netCDF4 as nc
from netCDF4 import Dataset
from threading import Thread


#passing parameters from sub menu

def execute(GNCDir, InFileName, WorkDir, OutFileName):
    if len(GNCDir) > 0 and len(InFileName) > 0 and len(WorkDir) > 0 and len(OutFileName) > 0:

        progress['value'] = 10
          
        os.chdir(WorkDir)
        ilwis.setWorkingCatalog(WorkDir)
        
        #filename = rfe2025_03_anom.v3.1.nc
        for filename in glob.glob(os.path.join(GNCDir+'/rfe'+str(InFileName[:4])+'_'+str(InFileName[4:6])+'_anom.v*.nc')):
            shutil.copy(filename, WorkDir)

        progress['value'] = 20  

        for fnames in glob.glob(os.path.join(WorkDir+r'/rfe'+str(InFileName[:4])+'_'+str(InFileName[4:6])+'_anom.v*.nc')):
            file_in = fnames
            
        ds = nc.Dataset(file_in)
        data = ds['rfe']    
        
        data_2d = np.squeeze(data)
        
        progress['value'] = 60 
        
        #create numpy array from nc variable
        tamsat = data_2d[:]
        tamsat.shape
        
        grf = ilwis.GeoReference('code=georef:type=corners, csy=epsg:4326, envelope=-19 38 52 -36, gridsize=1894 1974, cornerofcorners=yes')
        dfNum = ilwis.DataDefinition(ilwis.NumericDomain('code=value'), ilwis.NumericRange(-1500, 1500, 0))
        rc = ilwis.RasterCoverage()
        rc.setSize(ilwis.Size(1894,1974,1))
        rc.setGeoReference(grf)
        rc.setDataDef(dfNum)
        
        # change array to ilwis array
        rc.array2raster(tamsat)

        progress['value'] = 80
        
        rcNew = ilwis.do('mapcalc', 'iff(@1==-9999, ?, @1)',rc) 
        rcNew.store(WorkDir+'/'+OutFileName+'.mpr')
        
        progress['value'] = 100
        
        button0.config(state='normal')
        button1.config(state='normal')

#redundant files deleted in main menu script

def executeInThread(GNCDir, InFileName, WorkDir, OutFileName):
    button0.config(state='disabled')
    button1.config(state='disabled')
    thread = Thread(target = execute, args = (GNCDir, InFileName, WorkDir, OutFileName))
    thread.start()



# for command=doNothing
def doNothing():
   text="Do nothing button"
   return


# for command=clickthumbn
def clickthumbn():
   novi = Toplevel(app)
   novi.title ('Example of imported TAMSAT Monthly Precipitation Anomaly image (in ILWIS)')
   novi.iconbitmap(gnc_path +'\pics\gnc.ico')
   canvas = Canvas(novi, width = 382, height = 400)
   canvas.pack(expand = YES, fill = BOTH)
   gif1 = PhotoImage(file = gnc_path +'\pics\P3_TAMSAT_Monthly_anom.gif')
   canvas.create_image(0, 0,image = gif1, anchor = NW)
   canvas.gif1 = gif1

# for command=clickAbout
def clickAbout(): 
   filewin = Toplevel(app)
   filewin.title ('About TAMSAT')
   filewin.iconbitmap(gnc_path +'\pics\gnc.ico')
   filewin.geometry("575x340")
   msg = Message(filewin, text="""TAMSAT Precipitation background:

Rainfall Estimate for Africa - MSG. The TAMSAT rainfall estimates are based on Meteosat thermal infrared imagery, calibrated against ground-based rain gauge measurements, and cover all of Africa. The products also include rainfall anomalies. They are available in different frequencies: daily, pentadal, dekadal and monthly.

File name convertion: rfe2025_03_anom.v3.1.nc
Number of segments: 1 file 
Temporal Resolution: monthly resolution
Initial file format: NetCDF

Data is provided throught EUMETCast reception. For Legal Constraints check the 'Product Navigator' at EUMETSAT available at http://navigator.eumetsat.int
""")
   msg.pack()


#main()
#automatically setting of path when running this script from main menu - uncomment to make active   
gnc_path = os.environ["gnc_popup"]

#check path
print (gnc_path)
app = Tk()
app.title("TAMSAT import routine")
app.iconbitmap(gnc_path +'\pics\gnc.ico')

w = 650 # width for the Tk app
h = 210 # height for the Tk app

app.geometry('%dx%d' % (w, h))
app.resizable(0,0)

labelText1 = StringVar()
labelText1.set ("""TAMSAT Monthly Anomaly
Typical filename: rfe2025_03_anom.v3.1.nc""")

label1 = Label(app, textvariable=labelText1, anchor='w', justify = LEFT)
label1.pack(side = LEFT)
label1.place(x = 10, y = 5, width=650, height=35)

L1 = Label(app, text="Input Directory:", anchor='w')
L1.pack(side = LEFT)
L1.place(x = 10, y = 50, width=150, height=25)

inputDir = StringVar()
E1 = Entry(app, textvariable=inputDir, bd =2)
E1.pack()
E1.place(x = 165, y = 50, width=445, height=25)

button_dirin = Button(app, text="....", width=2, command = lambda: inputDir.set(askdirectory(initialdir = inputDir.get())))
button_dirin.pack()
button_dirin.place(x = 615, y = 49)

L2 = Label(app, text="Time Stamp (in UTC-time):", anchor='w')
L2.pack(side = LEFT)
L2.place(x = 10, y = 80, width=150, height=25)

inputFile = StringVar()
E2 = Entry(app, textvariable=inputFile, bd =2)
E2.pack()
E2.place(x = 165, y = 80, width=445, height=25)

#button_filein = Button(app, text="....", width=2, command = lambda: inputFile.set(askopenfilename(initialdir = inputDir.get()).split('/')[-1]))
#button_filein.pack()
#button_filein.place(x = 615, y = 79)

L3 = Label(app, text="Output Directory:", anchor='w')
L3.pack(side = LEFT)
L3.place(x = 10, y = 110, width=150, height=25)

outputDir = StringVar()
E3 = Entry(app, textvariable=outputDir, bd =2)
E3.pack()
E3.place(x = 165, y = 110, width=445, height=25)

button_dirout = Button(app, text="....", width=2, command = lambda: outputDir.set(askdirectory(initialdir = outputDir.get())))
button_dirout.pack()
button_dirout.place(x = 615, y = 109) 

L4 = Label(app, text="Output Filename:", anchor='w')
L4.pack(side = LEFT)
L4.place(x = 10, y = 140, width=150, height=25)

outputFile = StringVar()
E4 = Entry(app, textvariable=outputFile, bd =2)
E4.pack()
E4.place(x = 165, y = 140, width=445, height=25)

button0 = Button(app, text="Quit", width=15, command=app.destroy)
button0.pack()
button0.place(x = 525, y = 180)

button1 = Button(app, text="Execute", width=15, command=lambda: executeInThread(inputDir.get(), inputFile.get(), outputDir.get(), outputFile.get()))
button1.pack()
button1.place(x = 405, y = 180)

button2 = Button(app, text="About RFE", width=15, command=clickAbout)
button2.pack()
button2.place(x = 285, y = 180)

button3 = Button(app, text="Thumbnail", width=15, command=clickthumbn)
button3.pack()
button3.place(x = 165, y = 180)


style = Style()
style.theme_use('clam')
style.configure('blue.Horizontal.TProgressbar', foreground='blue', background='blue')

progress = Progressbar(app, style='blue.Horizontal.TProgressbar', orient=HORIZONTAL, length=100, mode='determinate')
progress.pack()
progress.place(x = 10, y = 180)


app.mainloop()

print("inputFile=" + inputFile.get())
print("outputDir=" + outputDir.get())
print("outputFile=" + outputFile.get())



